#include <list>
#include <sstream>
#include <string>
#include <unordered_map>
#include <stdexcept>
#include <algorithm>
#include <ctype.h>

#pragma hdrstop

#include "Graphics.h"
#include "Logic.h"
#include "ClientApp.h"

static const char *s_ppszMessages[] = {
  "OK",
  "Write city from.",
  "Write city to.",
  "Write leave time.",
  "City from not found.",
  "City to not found.",
  "City not found.",
  "Wrong date/time format.",
  "Bus not found.",
  "No buses found.",
  "Write number of seats.",
  "No bus set. Use BUS command.",
  "No customer number set. Use NEW command.",
  "Not so many free seats.",
  "Write list of seats to reserve.",
  "Cannot reserve required seats.",
  "Cannot unlock all required seats.",
  "Write bus line.",
  "Write bus plan.",
  "Bus plan not found.",
  "Write bus number.",
  "Cannot delete bus.",
  "Write city name.",
  "Too many arguments. Unused ones are ignored.",
  "Cannot add city.",
  "Cannot delete city.",
  "Cannot add city alias.",
  "Write new alias and existing city name.",
  "Write name of bus plan.",
  "Write number of seats.",
  "Cannot add bus plan.",
  "Cannot delete bus plan.",
  "Write proper bus plan file.",
  "Cannot add bus.",
  "Wrong bus number.",
  "Use FIND first.",
  "Write customer number.",
  "Write what to clear.",
  "Wrong argument.",
  "Cannot read the file.",
  "Reservations disabled in this bus.",
  "Cannot delete this bus. Wait until it leaves or disable it first.",
  "Cannot disable bus."
};

const char *CClientApp::Messages(Error eNumber) const
{
  return s_ppszMessages[static_cast<unsigned>(eNumber)];
}

CClientApp::CClientApp(CQuery *cQuery, bool fIsFull) : m_cLogic(CLogic(cQuery)), m_fIsFull(fIsFull)
{
  m_cCommands.reserve(20);
  m_cCommands.insert(std::make_pair("find", &CClientApp::Find));
  m_cCommands.insert(std::make_pair("next", &CClientApp::Next));
  m_cCommands.insert(std::make_pair("bus", &CClientApp::Bus));
  m_cCommands.insert(std::make_pair("reserve", &CClientApp::Reserve));
  m_cCommands.insert(std::make_pair("unlock", &CClientApp::Unlock));
  m_cCommands.insert(std::make_pair("city", &CClientApp::City));
  if(fIsFull) {
    m_cCommands.insert(std::make_pair("addbus", &CClientApp::AddBus));
    m_cCommands.insert(std::make_pair("delbus", &CClientApp::DelBus));
    m_cCommands.insert(std::make_pair("addcity", &CClientApp::AddCity));
    m_cCommands.insert(std::make_pair("delcity", &CClientApp::DelCity));
    m_cCommands.insert(std::make_pair("addcityalias", &CClientApp::AddCityAlias));
    m_cCommands.insert(std::make_pair("addbusplan", &CClientApp::AddBusPlan));
    m_cCommands.insert(std::make_pair("delbusplan", &CClientApp::DelBusPlan));
    m_cCommands.insert(std::make_pair("clear", &CClientApp::Clear));
    m_cCommands.insert(std::make_pair("close", &CClientApp::Close));
  }
}

CClientApp::~CClientApp()
{
}

bool CClientApp::Run(void)
{
  m_cGraphics.WriteWelcome();
  do {
    // chrnn blok pro zachycen vjimek zpsobench chybou komunikace se serverem
    try {
      m_cLogic.Init();
      while(true) {
        // nati dek ze vstupu
        while(m_strInput.empty())
          if(!m_cGraphics.Read(m_strInput))
            return true; // nap. pi EOF ze souboru pesmrovanho na vstup
        std::string strComm = GetOneParam(); // vyzvednut pkazu
    	  std::transform(strComm.begin(), strComm.end(), strComm.begin(), tolower); // pevod na mal psmena
        if(strComm == "exit")
          return true;
        else if(strComm == "help")
          m_cGraphics.WriteHelp(m_fIsFull);
        else
          ProcessCommand(strComm); // zpracovn pkazu
        m_strInput.clear();
      }
    }
    catch(std::runtime_error &cExc) { // vypsn piny vjimky
      m_cGraphics.Write("Fatal error:");
      m_cGraphics.Write(cExc.what());
    }
  } while(m_cGraphics.Continue()); // dal pokus po chyb a potvrzen uivatelem
  return false;
}

std::string CClientApp::GetOneParam(void)
{
  std::istringstream cInput(m_strInput);
  std::string strParam;
  cInput >> strParam;
  if(strParam[0] == '"' && strParam.length() > 1) {
    strParam = strParam.substr(1);
    while(strParam[strParam.length() - 1] != '"') {
      std::string strTemp;
      cInput >> strTemp;
      strParam += " ";
      strParam += strTemp;
    }
    strParam = strParam.substr(0, strParam.length() - 1);
    m_strInput = m_strInput.substr(m_strInput.find(strParam) + strParam.length()+1);
  }
  else
    m_strInput = m_strInput.substr(m_strInput.find(strParam) + strParam.length());
  return strParam;
}

unsigned CClientApp::GetNumber(std::string strNumber)
{
  std::istringstream cInput(strNumber);
  unsigned uiNumber;
  cInput >> uiNumber;
  return uiNumber;
}

bool CClientApp::GetList(std::list<unsigned> *cSeats, const std::string& strParam)
{
  unsigned uiId;
  char cChar;
  std::istringstream cInput(strParam);
  cSeats->clear();
  if(!(cInput >> uiId))
    return false; // przdn nebo neplatn vstup
  cSeats->push_back(uiId); // prvn slo
  while(cInput >> cChar && cChar == ',') {
    if(cInput >> uiId)
      cSeats->push_back(uiId); // dal sla oddlen rkami
    else {
      cSeats->clear();
      return false; // nepodailo se nast slo
    }
  }
  if(!cInput.eof()) { // na vstupu zstaly neplatn znaky
    cSeats->clear();
    return false;
  }
  return true;
}

void CClientApp::ProcessCommand(std::string strComm)
{
  auto i = m_cCommands.find(strComm);
  if(i == m_cCommands.end())
    m_cGraphics.Write("Unknown command.");
  else
    (this->*(i->second))(); // zavoln metody pomoc itertoru, tj. ukazatel na n je i->second
}

// --- metody typu FnCommand ---

void CClientApp::Find(void)
{
  std::list<RBusPrint> cRBusPrint;
  std::string strCityFrom = GetOneParam();
  std::string strCityTo = GetOneParam();
  std::string strLeaveTime = GetOneParam();
  if(!m_cLogic.Find(&cRBusPrint, strCityFrom.c_str(), strCityTo.c_str(), strLeaveTime.c_str()))
    m_cGraphics.Write(Messages(m_cLogic.GetLastError()));
  if(!cRBusPrint.empty())
    m_cGraphics.WriteBuses(&cRBusPrint);
}

void CClientApp::Next(void)
{
  std::list<RBusPrint> cRBusPrint;
  m_cLogic.Next(&cRBusPrint);
  if(!cRBusPrint.empty())
    m_cGraphics.WriteBuses(&cRBusPrint);
  else
    m_cGraphics.Write(Messages(m_cLogic.GetLastError()));
}

void CClientApp::Bus(void)
{
  std::string strBusPlan;
  std::list<unsigned> cSeats;
  RBusPrint rBusPrint;
  unsigned uiNumberSeats;
  bool fOk = m_cLogic.SetBus(GetNumber(GetOneParam()), &cSeats, &strBusPlan, &rBusPrint, &uiNumberSeats);
  m_cGraphics.Write(Messages(m_cLogic.GetLastError()));
  if(fOk)
    m_cGraphics.WriteBusPlan(strBusPlan, &cSeats, &rBusPrint, uiNumberSeats);
}

void CClientApp::Reserve(void)
{
  std::string strNumber;
  std::list<unsigned> cSeats;
  RBusPrint rBusPrint;
  strNumber = GetOneParam();
  // nejprve pedbn rezervace
  if(strNumber[0] == '#') { // rezervace potu sedadel (vbr na programu)
    strNumber = strNumber.substr(1);
    unsigned uiSeats = GetNumber(strNumber);
    if(!m_cLogic.ReserveAsk(&cSeats, uiSeats)) {
      m_cGraphics.Write(Messages(m_cLogic.GetLastError()));
      return;
    }
  }
  else { // rezervace vybranch sedadel (ureno zkaznkem)
    GetList(&cSeats, strNumber);
    if(!m_cLogic.ReserveAsk(&cSeats)) {
      m_cGraphics.Write(Messages(m_cLogic.GetLastError()));
      m_cLogic.Unlock(&cSeats);
      return;
    }
  }
  // konen rezervace po potvrzen zkaznkem
  m_cGraphics.WriteSeats("Seats to be reserved: ", &cSeats);
  if(m_cGraphics.Continue()) {
    unsigned uiCustomer;
    if(m_cLogic.ReserveForever(&cSeats, &uiCustomer, &rBusPrint)) {
      m_cGraphics.Write(Messages(m_cLogic.GetLastError()));
      m_cGraphics.WriteReservation(uiCustomer, &cSeats, &rBusPrint);
    }
    else
      m_cGraphics.Write(Messages(m_cLogic.GetLastError()));
  }
  else { // nepotvrzeno
    if(m_cLogic.Unlock(&cSeats))
      m_cGraphics.Write("Reservation canceled.");
    else
      m_cGraphics.Write(Messages(m_cLogic.GetLastError()));
  }
}

void CClientApp::Unlock(void)
{
  std::string strNumber = GetOneParam();
  if(strNumber.empty()) {
    m_cGraphics.Write(Messages(WRUSERID));
    return;
  }
  unsigned uiId = GetNumber(strNumber);
  std::string strTemp = GetOneParam();
  std::list<unsigned> cSeats;
  if(strTemp.empty())
    m_cLogic.UnlockAll(uiId, &cSeats);
  else {
    GetList(&cSeats, strTemp);
    m_cLogic.Unlock(uiId, &cSeats);
  }
  m_cGraphics.Write(Messages(m_cLogic.GetLastError()));
  m_cGraphics.WriteSeats("Unlocked seats: ", &cSeats);
}

void CClientApp::AddBus(void)
{
  std::string strLine = GetOneParam();
  std::string strCityFrom = GetOneParam();
  std::string strCityTo = GetOneParam();
  std::string strLeaveTime = GetOneParam();
  std::string strPlan = GetOneParam();
  unsigned uiId = m_cLogic.AddBus(GetNumber(strLine), strCityFrom.c_str(), strCityTo.c_str(), strLeaveTime.c_str(), strPlan.c_str());
  if(uiId == 0)
    m_cGraphics.Write(Messages(m_cLogic.GetLastError()));
  else
    m_cGraphics.WriteAddBus(uiId);
}
  
void CClientApp::DelBus(void)
{
  m_cLogic.DelBus(GetNumber(GetOneParam()));
  m_cGraphics.Write(Messages(m_cLogic.GetLastError()));
}
   
void CClientApp::AddCity(void)
{
  unsigned uiId = m_cLogic.AddCity(GetOneParam().c_str());
  if(uiId == 0)
    m_cGraphics.Write(Messages(m_cLogic.GetLastError()));
  else
    m_cGraphics.WriteAddCity(uiId);
}
  
void CClientApp::DelCity(void)
{
  m_cLogic.DelCity(GetOneParam().c_str());
  m_cGraphics.Write(Messages(m_cLogic.GetLastError()));
}
   
void CClientApp::AddCityAlias(void)
{
  std::string strAliasName = GetOneParam();
  std::string strCityName = GetOneParam();
  m_cLogic.AddCityAlias(strAliasName.c_str(), strCityName.c_str());
  m_cGraphics.Write(Messages(m_cLogic.GetLastError()));
}
   
void CClientApp::AddBusPlan(void)
{
  std::string strPlan = GetOneParam();
  std::string strSeatsCount = GetOneParam();
  std::string strFile = GetOneParam();
  unsigned uiId = m_cLogic.AddBusPlan(strPlan.c_str(), GetNumber(strSeatsCount), strFile.c_str());
  if(uiId == 0)
    m_cGraphics.Write(Messages(m_cLogic.GetLastError()));
  else
    m_cGraphics.WriteAddPlan(uiId);
}
   
void CClientApp::DelBusPlan(void)
{
  m_cLogic.DelBusPlan(GetOneParam().c_str());
  m_cGraphics.Write(Messages(m_cLogic.GetLastError()));
}

void CClientApp::Clear(void)
{
  std::string strParam = GetOneParam();
  if(strParam.empty()) {
    m_cGraphics.Write(Messages(NTYPESET));
    return;
  }
  std::transform(strParam.begin(), strParam.end(), strParam.begin(), tolower); // pevod na mal psmena
  if(strParam == "bus")
    m_cGraphics.WriteDeleted(m_cLogic.ClearBus());
  else if(strParam == "city")
    m_cGraphics.WriteDeleted(m_cLogic.ClearCity());
  else if(strParam == "plan")
    m_cGraphics.WriteDeleted(m_cLogic.ClearPlan());
  else
    m_cGraphics.Write(Messages(WRARGUMENT));
}

void CClientApp::Close(void)
{
  std::string strParam = GetOneParam();
  if(strParam.empty()) {
    m_cGraphics.Write(Messages(WRITEBID));
    return;
  }
  std::list<unsigned> cReservedSeats;
  unsigned uiSeats;
  RBusPrint rBus;
  std::string strPlan;
  bool f = m_cLogic.Close(GetNumber(strParam), &cReservedSeats, &strPlan, &rBus, &uiSeats);
  m_cGraphics.Write(Messages(m_cLogic.GetLastError()));
  if(f) {
    m_cGraphics.WriteBusPlan(strPlan, &cReservedSeats, &rBus, uiSeats, "Reserved seats: ");
  }
}

void CClientApp::City(void)
{
  std::string strParam = GetOneParam();
  std::vector<std::pair<std::string, std::string>> cOut; //prvni je cely nazev, druhy je alias
  m_cLogic.City(strParam, cOut);
  if(!cOut.empty())
    m_cGraphics.WriteCities(cOut);
  else
    m_cGraphics.Write(Messages(CTNFOUND));
}
