#ifndef _CLIENTAPP_H
#define _CLIENTAPP_H

#include <list>
#include <unordered_map>
#include <string>
#include "Graphics.h"
#include "Logic.h"

/*!
@file ClientApp.h
Deklarace hlavn tdy klientsk aplikace @ref CClientApp.
*/

class CClientApp;
//! typ metody pro zpracovn pkazu
/*!
Jednotliv metody jsou zpracovvny metodami tdy @ref CClientApp. Penos daj 
se dje pomoc lenskch promnnch tdy, proto metody nemaj dn parametry a 
nvratov hodnota je typu @c void.
*/
typedef void (CClientApp::*FnCommand)(void);

//! hlavn tda klientsk aplikace
/*!
Hlavn tda klientsk aplikace zabezpeuje vstup pkaz od uivatele (k tomu 
si zizuje privtn instanci @ref CGraphics), rozpoznn pkaz vetn vyzvednut 
a syntaktick kontroly ppadnch parametr. Dle vol zpracovn pkazu pomoc 
modulu klientsk logiky (privtn instance tdy @ref CLogic). Vsledky zpracovn 
i ppadn chybov stavy zobrazuje obsluze opt pomoc @ref CGraphics.

O zpracovn kadho pkazu se star pslun privtn metoda tdy typu
@ref FnCommand. Pro rychl vyhledvn metody odpovdajc danmu pkazu tda 
obsahuje privtn hashovac tabulku pro piazen @e pkaz-metoda. To umouje i 
snadn pidvn dalch pkaz v ppad poteby.
*/
class CClientApp {
public:
  //! konstruktor
  /*!
  Vytvoen instance tdy, je tvo zklad klientsk aplikace. 
  @param[in] cQuery ukazatel na instanci tdy, kter implementuje databzov 
  rozhran. V ppad klientsk aplikace to je typicky modul, kter zprostedkovv 
  penos pkaz na server pomoc @ref ipc "meziprocesov komunikace". Parametr 
  potebuje k tomu, aby privtn vytvoen instance tdy @ref CLogic mohla bt 
  pipojena k odkazovanmu databzovmu rozhran.
  @param[in] fIsFull pepna, zda je tento klient s plnm oprvnnm (pro vedoucho
  dispeera) nebo zda m jen redukovan oprvnn (pro pokladn za pepkou).
  */
  CClientApp(CQuery *cQuery, bool fIsFull);
  //! destruktor
  ~CClientApp();
  //! sputn zpracovatelsk smyky
  /*!
  Vstup do smyky 
  @li vyzvednut pkazu, jeho rozeznn a kontrola parametr,
  @li pedn ke zpracovn,
  @li vyzvednut vsledk a jejich prezentace.

  Smyka je ukonena pkazem @c EXIT (ppadn koncem souboru, jedn-li se 
  o vstup ze souboru pesmrovanho na standardn vstup).

  V ppad zvan chyby (nedostupn nebo chybn nakonfigurovan server, 
  ztrta dat pi komunikaci, atp.) je vypsno oznmen zvan chyby. Obsluha 
  se me snait najt zdroj problmu a po odstrann opakovat pokus nebo
  zpracovn ve smyce ukonit.
  @return hodnota @e true pi regulernm ukonen zpracovatelsk smyky,
  @e false pi ukonen nsledkem zvan chyby komunikace se serverem.
  */
  bool Run(void);
private:
  CLogic m_cLogic;
  CGraphics m_cGraphics;
  std::string m_strInput;
  std::unordered_map<std::string, FnCommand> m_cCommands;
  std::string GetOneParam(void);
  unsigned GetNumber(std::string strNumber);
  bool GetList(std::list<unsigned> *cSeats, const std::string& strParam);
  void ProcessCommand(std::string strComm);
  const char *Messages(Error eNumber) const;
  bool m_fIsFull;
  // metody typu FnCommand
  void Find(void);
  void Next(void);
  void Bus(void);
  void Reserve(void);
  void Unlock(void);
  void AddBus(void);
  void DelBus(void);
  void AddCity(void);
  void DelCity(void);
  void AddCityAlias(void);
  void AddBusPlan(void);
  void DelBusPlan(void);
  void Clear(void);
  void Close(void);
  void City(void);
};

#endif
