#ifndef _CLIENTCONN_H
#define _CLIENTCONN_H

/*!
@file ClientConn.h
Deklarace tdy @ref CClientConnection a nvratovch hodnot metody @ref CClientConnection::Process.
*/

class CCommand;

//! klientsk st komunikanho rozhran
/*!
Tda slou jako spolen rozhran pro klientskou st komunikace, nezvisle na konkrtnm
zpsobu transportu pkaz. Konkrtn realizace mus bt dokonena v potomcch tohoto rozhran.

Klient me pkazy vykonvat ve dvou reimech:
@li jednotliv - viz metoda @ref Process,
@li dvkov v rmci sezen (session). Sezen je poteba nejprv otevt, pak je mon provdt jednotliv 
pkazy dle poteby a nakonec se sezen uzave (viz @ref OpenSession, @ref SessionCmd a @ref CloseSession).
*/
class CClientConnection {
public:
  //! konstruktor
  /*! @param[in] uiTimeout maximln doba v milisekundch, po kterou klient ek na odpov od serveru */
  CClientConnection(unsigned uiTimeout);
  //! destruktor
  virtual ~CClientConnection();
  //! oteven sezen
  /*!
  Provede se pipojen k serveru a inicializace sezen. Ppadn oteven pedchoz sezen je ukoneno.
  @return vsledek - nenulov hodnota zna chybu @n Seznam chybovch kd s vysvtlenm @ref ClientConn.h "zde".
  */
  int OpenSession(void);
  //! vykonn pkazu v rmci sezen
  /*!
  Klient pomoc chrnnch metod postupn provede nsledujc operace:
  @li odele pkaz na server
  @li po dobu urenho asovho limitu ek na odpov
  @li pijme odpov a pslun ji zformtuje
  @param[in, out] cCmd tda @ref CCommand obsahujc data pkazu a stavov slovo
  @param[in] uiAddTimeout dodaten asov limit pro vykonn pkazu v milisekundch (pit se 
  k zkladn hodnot definovan v kontruktoru tdy)
  @return vsledek - nenulov hodnota zna chybu @n Seznam chybovch kd s vysvtlenm @ref ClientConn.h "zde".
  */
  int SessionCmd(CCommand *cCmd, unsigned uiAddTimeout = 0);
  //! uzaven sezen
  /*!
  Oteven sezen je uzaveno a spojen se serverem ukoneno.
  @return vsledek - nenulov hodnota zna chybu @n Seznam chybovch kd s vysvtlenm @ref ClientConn.h "zde".
  */
  int CloseSession(void);
  //! vykonn pkazu na serveru
  /*!
  Klient postupn provede nsledujc operace:
  @li pipoj se k serveru
  @li odele pkaz na server
  @li po dobu urenho asovho limitu ek na odpov
  @li pijme odpov a pslun ji zformtuje
  @li ukon spojen se serverem
  Pro vykonn jednotlivch operac vol pslun metody, kter jsou implementovny podle druhu transportu.
  @param[in, out] cCmd tda @ref CCommand obsahujc data pkazu a stavov slovo
  @param[in] uiAddTimeout dodaten asov limit pro vykonn pkazu v milisekundch (pit se 
  k zkladn hodnot definovan v kontruktoru tdy)
  @return vsledek - nenulov hodnota zna chybu @n Seznam chybovch kd s vysvtlenm @ref ClientConn.h "zde".
  Pokud je metoda vyvolna v dob aktivnho sezen, je po nvratu sezen ukoneno.
  */
  int Process(CCommand *cCmd, unsigned uiAddTimeout = 0);
protected:
  //! pipojen k serveru
  /*! @return spch operace */
  virtual bool ConnectServer(void) = 0;
  //! odpojen od serveru
  virtual void DisconnectServer(void) = 0;
  //! odesln pkazu
  /*!
  @param[in] cCmd pkaz k odesln
  @return spch operace
  */
  virtual bool Write(const CCommand *cCmd) = 0;
  //! ekn na odpov
  /*! @return @e true v ppad, e bhem poadovanho asovho limitu byla pijata odpov od serveru */
  virtual bool Wait(unsigned uiTimeout) = 0;
  //! vyzvednut odpovdi
  /*!
  @param[in] cCmd tda pro uloen pijatch dat a stavovho slova
  @return spch operace
  */
  virtual bool Read(CCommand *cCmd) = 0;
private:
  unsigned m_uiTimeout;
  bool m_fSession;
};

/*! spn dokoneno */
const int CMD_SUCCESS = 0;
/*! nepodailo se navzat spojen se serverem */
const int CMD_ERR_CONNECT = 1;
/*! chyba pi odesln dat na server */
const int CMD_ERR_WRITE = 2;
/*! chyba pi pjmu dat ze serveru */
const int CMD_ERR_READ = 3;
/*! dn odpov bhem asovho limitu */
const int CMD_ERR_TIMEOUT = 4;
/*! nen oteveno sezen */
const int CMD_ERR_NOSESSION = 5;

#endif
