#define _CRT_SECURE_NO_WARNINGS

#include <string>
#include <list>
#include <string.h>
#include <stdexcept>

#pragma hdrstop

#include "ClientQuery.h"
#include "../Common/Query.h"
#include "../Common/Command.h"
#include "ClientConn.h"

//-------------------------------------------------------------------------------------------------
// pomocn tda pro naplnn CCommand

class CCmdBuffer {
public:
  CCmdBuffer(CCommand::EQueryCmd eCmd, unsigned uiParamLen);
  ~CCmdBuffer();
  void Unsigned(unsigned uiValue, unsigned uiOffset);
  void String(const char *pszValue, unsigned uiOffset);
  void Block(const void *pData, unsigned uiSize, unsigned uiOffset);
  void *Data(void) const;
  unsigned Length(void) const;
  void Clear(void);
private:
  char *m_pcBuffer;
  unsigned m_uiLength;
};

CCmdBuffer::CCmdBuffer(CCommand::EQueryCmd eCmd, unsigned uiParamLen)
{
  unsigned uiCmd = eCmd; // uloeno do typu unsigned, protoe velikost vtovho typu je obecn promnn
  m_uiLength = sizeof(unsigned) + uiParamLen;
  m_pcBuffer = new char [m_uiLength];
  memcpy(m_pcBuffer, &uiCmd, sizeof(unsigned)); // pkaz vdy na zatku
}

CCmdBuffer::~CCmdBuffer()
{
  delete [] m_pcBuffer;
}

void CCmdBuffer::Unsigned(unsigned uiValue, unsigned uiOffset)
{
  memcpy(m_pcBuffer + sizeof(unsigned) + uiOffset, &uiValue, sizeof(unsigned));
}

void CCmdBuffer::String(const char *pszValue, unsigned uiOffset)
{
  strcpy(m_pcBuffer + sizeof(unsigned) + uiOffset, pszValue);
}

void CCmdBuffer::Block(const void *pData, unsigned uiSize, unsigned uiOffset)
{
  memcpy(m_pcBuffer + sizeof(unsigned) + uiOffset, pData, uiSize);
}

void *CCmdBuffer::Data(void) const
{
  return m_pcBuffer;
}

unsigned CCmdBuffer::Length(void) const
{
  return m_uiLength;
}

void CCmdBuffer::Clear(void)
{
  delete [] m_pcBuffer;
  m_pcBuffer = NULL;
}

//-------------------------------------------------------------------------------------------------
CClientQuery::CClientQuery(CClientConnection *cConnection)
{
  m_cConnection = cConnection;
  m_cCounter = 0;
}

CClientQuery::~CClientQuery()
{
}

unsigned CClientQuery::AddCity(const char *pszName)
{
  // vstupn parametry
  CCmdBuffer cCmdBuffer(CCommand::CMDADDCITY, static_cast<unsigned>(strlen(pszName)) + 1);
  cCmdBuffer.String(pszName, 0);
  CCommand cCmd(++m_cCounter & CMD_COUNTER_MASK, cCmdBuffer.Data(), cCmdBuffer.Length());
  cCmdBuffer.Clear();
  // vzdlen voln zpracovatelsk metody
  CallQuery(&cCmd);
  // nvratov hodnota
  return *(unsigned *)cCmd.DataPointer();
}

bool CClientQuery::AddCityAlias(const char *pszName, unsigned uiId)
{
  // vstupn parametry
  unsigned uiStrLen = static_cast<unsigned>(strlen(pszName)) + 1;
  CCmdBuffer cCmdBuffer(CCommand::CMDADDCITYALIAS, uiStrLen + sizeof(unsigned));
  cCmdBuffer.String(pszName, 0);
  cCmdBuffer.Unsigned(uiId, uiStrLen);
  CCommand cCmd(++m_cCounter & CMD_COUNTER_MASK, cCmdBuffer.Data(), cCmdBuffer.Length());
  cCmdBuffer.Clear();
  // vzdlen voln zpracovatelsk metody
  CallQuery(&cCmd);
  // nvratov hodnota
  return *(unsigned *)cCmd.DataPointer() != 0;
}

bool CClientQuery::DeleteCity(const char *pszName)
{
  // vstupn parametry
  CCmdBuffer cCmdBuffer(CCommand::CMDDELETECITY, static_cast<unsigned>(strlen(pszName)) + 1);
  cCmdBuffer.String(pszName, 0);
  CCommand cCmd(++m_cCounter & CMD_COUNTER_MASK, cCmdBuffer.Data(), cCmdBuffer.Length());
  cCmdBuffer.Clear();
  // vzdlen voln zpracovatelsk metody
  CallQuery(&cCmd);
  // nvratov hodnota
  return *(unsigned *)cCmd.DataPointer() != 0;
}

unsigned CClientQuery::FindCity(const char *pszName)
{
  // vstupn parametry
  CCmdBuffer cCmdBuffer(CCommand::CMDFINDCITY, static_cast<unsigned>(strlen(pszName)) + 1);
  cCmdBuffer.String(pszName, 0);
  CCommand cCmd(++m_cCounter & CMD_COUNTER_MASK, cCmdBuffer.Data(), cCmdBuffer.Length());
  cCmdBuffer.Clear();
  // vzdlen voln zpracovatelsk metody
  CallQuery(&cCmd);
  // nvratov hodnota
  return *(unsigned *)cCmd.DataPointer();
}

bool CClientQuery::CityInfo(unsigned uiCity, RCityInfo *prInfo, unsigned *puiLinesOut, unsigned *puiLinesIn)
{
  // vstupn parametry
  CCmdBuffer cCmdBuffer(CCommand::CMDCITYINFO, sizeof(unsigned));
  cCmdBuffer.Unsigned(uiCity, 0);
  CCommand cCmd(++m_cCounter & CMD_COUNTER_MASK, cCmdBuffer.Data(), cCmdBuffer.Length());
  cCmdBuffer.Clear();
  // vzdlen voln zpracovatelsk metody
  CallQuery(&cCmd);
  // nvratov hodnota
  bool fResult = *(unsigned *)cCmd.DataPointer() != 0;
  if(!fResult)
    return false;
  // vstupn parametry
  unsigned uiOffset = sizeof(unsigned);
  if(prInfo != NULL)
    cCmd.CopyData(&prInfo->uiId, sizeof(unsigned), uiOffset);
  uiOffset += sizeof(unsigned);
  if(prInfo != NULL)
    prInfo->strName = (const char *)cCmd.DataPointer() + uiOffset;
  uiOffset += static_cast<unsigned>(strlen((const char *)cCmd.DataPointer() + uiOffset) + 1);
  if(puiLinesOut != NULL)
    cCmd.CopyData(puiLinesOut, sizeof(unsigned), uiOffset);
  uiOffset += sizeof(unsigned);
  if(puiLinesIn != NULL)
    cCmd.CopyData(puiLinesIn, sizeof(unsigned), uiOffset);
  return true;
}

unsigned CClientQuery::AddBusPlan(const char *pszName, unsigned uiSeats, const CBlobEx<std::string> *cData)
{
  // vstupn parametry
  unsigned uiStrLen = static_cast<unsigned>(strlen(pszName) + 1);
  unsigned uiPlanStrLen = static_cast<unsigned>(cData->Get().length() + 1);
  unsigned uiPlanDataLen = cData->DataLen();
  CCmdBuffer cCmdBuffer(CCommand::CMDADDBUSPLAN, uiStrLen + sizeof(unsigned) + uiPlanStrLen + sizeof(unsigned) + uiPlanDataLen);
  cCmdBuffer.String(pszName, 0);
  cCmdBuffer.Unsigned(uiSeats, uiStrLen);
  cCmdBuffer.String(cData->Get().c_str(), uiStrLen + sizeof(unsigned));
  // velikost binrnch dat
  cCmdBuffer.Unsigned(uiPlanDataLen, uiStrLen + sizeof(unsigned) + uiPlanStrLen);
  // binrn data
  cCmdBuffer.Block(cData->DataPointer(), uiPlanDataLen, uiStrLen + sizeof(unsigned) + uiPlanStrLen + sizeof(unsigned));
  CCommand cCmd(++m_cCounter & CMD_COUNTER_MASK, cCmdBuffer.Data(), cCmdBuffer.Length());
  cCmdBuffer.Clear();
  // vzdlen voln zpracovatelsk metody
  CallQuery(&cCmd);
  // nvratov hodnota
  return *(unsigned *)cCmd.DataPointer();
}

bool CClientQuery::DeleteBusPlan(const char *pszName)
{
  // vstupn parametry
  CCmdBuffer cCmdBuffer(CCommand::CMDDELETEBUSPLAN, static_cast<unsigned>(strlen(pszName) + 1));
  cCmdBuffer.String(pszName, 0);
  CCommand cCmd(++m_cCounter & CMD_COUNTER_MASK, cCmdBuffer.Data(), cCmdBuffer.Length());
  cCmdBuffer.Clear();
  // vzdlen voln zpracovatelsk metody
  CallQuery(&cCmd);
  // nvratov hodnota
  return *(unsigned *)cCmd.DataPointer() != 0;
}

unsigned CClientQuery::FindBusPlan(const char *pszName)
{
  // vstupn parametry
  CCmdBuffer cCmdBuffer(CCommand::CMDFINDBUSPLAN, static_cast<unsigned>(strlen(pszName) + 1));
  cCmdBuffer.String(pszName, 0);
  CCommand cCmd(++m_cCounter & CMD_COUNTER_MASK, cCmdBuffer.Data(), cCmdBuffer.Length());
  cCmdBuffer.Clear();
  // vzdlen voln zpracovatelsk metody
  CallQuery(&cCmd);
  // nvratov hodnota
  return *(unsigned *)cCmd.DataPointer();
}

bool CClientQuery::BusPlanInfo(unsigned uiPlan, RPlanInfo *prInfo, unsigned *puiBuses, CBlobEx<std::string> *cData)
{
  // vstupn parametry
  CCmdBuffer cCmdBuffer(CCommand::CMDBUSPLANINFO, sizeof(unsigned));
  cCmdBuffer.Unsigned(uiPlan, 0);
  CCommand cCmd(++m_cCounter & CMD_COUNTER_MASK, cCmdBuffer.Data(), cCmdBuffer.Length());
  cCmdBuffer.Clear();
  // vzdlen voln zpracovatelsk metody
  CallQuery(&cCmd);
  // nvratov hodnota
  bool fResult = *(unsigned *)cCmd.DataPointer() != 0;
  if(!fResult)
    return false;
  // vstupn parametry
  unsigned uiOffset = sizeof(unsigned);
  if(prInfo != NULL)
    cCmd.CopyData(&prInfo->uiId, sizeof(unsigned), uiOffset);
  uiOffset += sizeof(unsigned);
  if(prInfo != NULL)
    prInfo->strName = (const char *)cCmd.DataPointer() + uiOffset;
  uiOffset += static_cast<unsigned>(strlen((const char *)cCmd.DataPointer() + uiOffset) + 1);
  if(prInfo != NULL)
    cCmd.CopyData(&prInfo->uiSeats, sizeof(unsigned), uiOffset);
  uiOffset += sizeof(unsigned);
  if(puiBuses != NULL)
    cCmd.CopyData(puiBuses, sizeof(unsigned), uiOffset);
  uiOffset += sizeof(unsigned);
  if(cData != NULL) {
    const char *pszPlanString = (const char *)cCmd.DataPointer() + uiOffset;
    const char *pszDataLen = pszPlanString + strlen(pszPlanString) + 1;
    const void *pData = pszDataLen + sizeof(unsigned);
    cData->Set(pszPlanString, pData, *(unsigned *)pszDataLen);
  }
  // nvratov hodnota
  return true;
}

unsigned CClientQuery::AddBus(const RBusInfo *prBus)
{
  // vstupn parametry
  CCmdBuffer cCmdBuffer(CCommand::CMDADDBUS, sizeof(RBusInfo));
  cCmdBuffer.Block(prBus, sizeof(RBusInfo), 0);
  CCommand cCmd(++m_cCounter & CMD_COUNTER_MASK, cCmdBuffer.Data(), cCmdBuffer.Length());
  cCmdBuffer.Clear();
  // vzdlen voln zpracovatelsk metody
  CallQuery(&cCmd);
  // nvratov hodnota
  return *(unsigned *)cCmd.DataPointer();
}

bool CClientQuery::DeleteBus(unsigned uiBus)
{
  // vstupn parametry
  CCmdBuffer cCmdBuffer(CCommand::CMDDELETEBUS, sizeof(unsigned));
  cCmdBuffer.Unsigned(uiBus, 0);
  CCommand cCmd(++m_cCounter & CMD_COUNTER_MASK, cCmdBuffer.Data(), cCmdBuffer.Length());
  cCmdBuffer.Clear();
  // vzdlen voln zpracovatelsk metody
  CallQuery(&cCmd);
  // nvratov hodnota
  return *(unsigned *)cCmd.DataPointer() != 0;
}

unsigned CClientQuery::FindBus(unsigned uiFrom, unsigned uiTo, time_t tLeaveTime)
{
  // vstupn parametry
  CCmdBuffer cCmdBuffer(CCommand::CMDFINDBUS, sizeof(unsigned) + sizeof(unsigned) + sizeof(time_t));
  cCmdBuffer.Unsigned(uiFrom, 0);
  cCmdBuffer.Unsigned(uiTo, sizeof(unsigned));
  cCmdBuffer.Block(&tLeaveTime, sizeof(time_t), sizeof(unsigned) + sizeof(unsigned));
  CCommand cCmd(++m_cCounter & CMD_COUNTER_MASK, cCmdBuffer.Data(), cCmdBuffer.Length());
  cCmdBuffer.Clear();
  // vzdlen voln zpracovatelsk metody
  CallQuery(&cCmd);
  // nvratov hodnota
  return *(unsigned *)cCmd.DataPointer();
}

bool CClientQuery::BusInfo(unsigned uiBus, RBusInfo *prInfo, bool *pfEnabled, unsigned *puiSeats, std::list<unsigned> *cFreeSeats)
{
  // vstupn parametry
  CCmdBuffer cCmdBuffer(CCommand::CMDBUSINFO, sizeof(unsigned));
  cCmdBuffer.Unsigned(uiBus, 0);
  CCommand cCmd(++m_cCounter & CMD_COUNTER_MASK, cCmdBuffer.Data(), cCmdBuffer.Length());
  cCmdBuffer.Clear();
  // vzdlen voln zpracovatelsk metody
  CallQuery(&cCmd);
  // nvratov hodnota
  bool fResult = *(unsigned *)cCmd.DataPointer() != 0;
  if(!fResult)
    return false;
  // vstupn parametry
  unsigned uiOffset;
  unsigned uiItems;
  unsigned uiValue;
  uiOffset = sizeof(unsigned);
  if(prInfo != NULL)
    cCmd.CopyData(prInfo, sizeof(RBusInfo), uiOffset); // agregt jednoduchch typ lze koprovat pmo
  uiOffset += sizeof(RBusInfo);
  if(pfEnabled != NULL) {
    cCmd.CopyData(&uiValue, sizeof(unsigned), uiOffset);
    *pfEnabled = uiValue != 0;
  }
  uiOffset += sizeof(unsigned);
  if(puiSeats != NULL)
    cCmd.CopyData(puiSeats, sizeof(unsigned), uiOffset);
  // list - poet poloek, data
  uiOffset += sizeof(unsigned);
  if(cFreeSeats != NULL) {
    cFreeSeats->clear();
    cCmd.CopyData(&uiItems, sizeof(unsigned), uiOffset);
    uiOffset += sizeof(unsigned);
    while(uiItems-- != 0) {
      cCmd.CopyData(&uiValue, sizeof(unsigned), uiOffset);
      cFreeSeats->push_back(uiValue);
      uiOffset += sizeof(unsigned);
    }
  }
  return true;
}

bool CClientQuery::EnableBus(unsigned uiBus)
{
  // vstupn parametry
  CCmdBuffer cCmdBuffer(CCommand::CMDENABLEBUS, sizeof(unsigned));
  cCmdBuffer.Unsigned(uiBus, 0);
  CCommand cCmd(++m_cCounter & CMD_COUNTER_MASK, cCmdBuffer.Data(), cCmdBuffer.Length());
  cCmdBuffer.Clear();
  // vzdlen voln zpracovatelsk metody
  CallQuery(&cCmd);
  // nvratov hodnota
  return *(unsigned *)cCmd.DataPointer() != 0;
}

bool CClientQuery::DisableBus(unsigned uiBus)
{
  // vstupn parametry
  CCmdBuffer cCmdBuffer(CCommand::CMDDISABLEBUS, sizeof(unsigned));
  cCmdBuffer.Unsigned(uiBus, 0);
  CCommand cCmd(++m_cCounter & CMD_COUNTER_MASK, cCmdBuffer.Data(), cCmdBuffer.Length());
  cCmdBuffer.Clear();
  // vzdlen voln zpracovatelsk metody
  CallQuery(&cCmd);
  // nvratov hodnota
  return *(unsigned *)cCmd.DataPointer() != 0;
}

unsigned CClientQuery::NewCustomer(void)
{
  // vstupn parametry
  CCmdBuffer cCmdBuffer(CCommand::CMDNEWCUSTOMER, 0);
  CCommand cCmd(++m_cCounter & CMD_COUNTER_MASK, cCmdBuffer.Data(), cCmdBuffer.Length());
  cCmdBuffer.Clear();
  // vzdlen voln zpracovatelsk metody
  CallQuery(&cCmd);
  // nvratov hodnota
  return *(unsigned *)cCmd.DataPointer();
}

bool CClientQuery::AskSeats(unsigned uiBus, std::list<unsigned> *cSeats, unsigned uiCustomer)
{
  return SeatOperation(CCommand::CMDASKSEATS, uiBus, cSeats, uiCustomer);
}

bool CClientQuery::LockSeats(unsigned uiBus, std::list<unsigned> *cSeats, unsigned uiCustomer)
{
  return SeatOperation(CCommand::CMDLOCKSEATS, uiBus, cSeats, uiCustomer);
}

bool CClientQuery::UnlockSeats(unsigned uiBus, std::list<unsigned> *cSeats, unsigned uiCustomer)
{
  return SeatOperation(CCommand::CMDUNLOCKSEATS, uiBus, cSeats, uiCustomer);
}

bool CClientQuery::SeatOperation(CCommand::EQueryCmd eCmd, unsigned uiBus, std::list<unsigned> *cSeats, unsigned uiCustomer)
{
  unsigned uiOffset;
  // vstupn parametry
  unsigned uiListItems = static_cast<unsigned>(cSeats->size());
  unsigned uiListSize = (uiListItems + 1) * sizeof(unsigned);
  CCmdBuffer cCmdBuffer(eCmd, sizeof(unsigned) + uiListSize + sizeof(unsigned));
  // slo autobusu
  cCmdBuffer.Unsigned(uiBus, 0);
  // seznam sedadel
  cCmdBuffer.Unsigned(uiListItems, sizeof(unsigned)); // poet
  uiOffset = sizeof(unsigned) + sizeof(unsigned);
  for(std::list<unsigned>::iterator i = cSeats->begin(); i != cSeats->end(); ++i) {
    cCmdBuffer.Unsigned(*i, uiOffset); // sedadlo
    uiOffset += sizeof(unsigned);
  }
  // slo zkaznka
  cCmdBuffer.Unsigned(uiCustomer, uiOffset);
  CCommand cCmd(++m_cCounter & CMD_COUNTER_MASK, cCmdBuffer.Data(), cCmdBuffer.Length());
  cCmdBuffer.Clear();
  // vzdlen voln zpracovatelsk metody
  CallQuery(&cCmd);
  // vstupn parametry - list: poet poloek + data
  unsigned uiItems;
  unsigned uiValue;
  uiOffset = sizeof(unsigned);
  cSeats->clear();
  cCmd.CopyData(&uiItems, sizeof(unsigned), uiOffset);
  uiOffset += sizeof(unsigned);
  while(uiItems-- != 0) {
    cCmd.CopyData(&uiValue, sizeof(unsigned), uiOffset);
    cSeats->push_back(uiValue);
    uiOffset += sizeof(unsigned);
  }
  // nvratov hodnota
  return *(unsigned *)cCmd.DataPointer() != 0;
}

unsigned CClientQuery::NextBus(unsigned uiBus)
{
  // vstupn parametry
  CCmdBuffer cCmdBuffer(CCommand::CMDNEXTBUS, sizeof(unsigned));
  cCmdBuffer.Unsigned(uiBus, 0);
  CCommand cCmd(++m_cCounter & CMD_COUNTER_MASK, cCmdBuffer.Data(), cCmdBuffer.Length());
  cCmdBuffer.Clear();
  // vzdlen voln zpracovatelsk metody
  CallQuery(&cCmd);
  // nvratov hodnota
  return *(unsigned *)cCmd.DataPointer();
}

unsigned CClientQuery::NextCity(unsigned uiCity)
{
  // vstupn parametry
  CCmdBuffer cCmdBuffer(CCommand::CMDNEXTCITY, sizeof(unsigned));
  cCmdBuffer.Unsigned(uiCity, 0);
  CCommand cCmd(++m_cCounter & CMD_COUNTER_MASK, cCmdBuffer.Data(), cCmdBuffer.Length());
  cCmdBuffer.Clear();
  // vzdlen voln zpracovatelsk metody
  CallQuery(&cCmd);
  // nvratov hodnota
  return *(unsigned *)cCmd.DataPointer();
}

unsigned CClientQuery::NextPlan(unsigned uiPlan)
{
  // vstupn parametry
  CCmdBuffer cCmdBuffer(CCommand::CMDNEXTPLAN, sizeof(unsigned));
  cCmdBuffer.Unsigned(uiPlan, 0);
  CCommand cCmd(++m_cCounter & CMD_COUNTER_MASK, cCmdBuffer.Data(), cCmdBuffer.Length());
  cCmdBuffer.Clear();
  // vzdlen voln zpracovatelsk metody
  CallQuery(&cCmd);
  // nvratov hodnota
  return *(unsigned *)cCmd.DataPointer();
}

bool CClientQuery::ListCity(const char *pszFrom, std::string *strName){
  // vstupn parametry
  CCmdBuffer cCmdBuffer(CCommand::CMDLISTCITY, static_cast<unsigned>(strlen(pszFrom) + 1));
  cCmdBuffer.String(pszFrom, 0);
  CCommand cCmd(++m_cCounter & CMD_COUNTER_MASK, cCmdBuffer.Data(), cCmdBuffer.Length());
  cCmdBuffer.Clear();
  // vzdlen voln zpracovatelsk metody
  CallQuery(&cCmd);
  // nvratov hodnota
  bool fResult = *(unsigned *)cCmd.DataPointer() != 0;
  if(!fResult)
    return false;
  // vstupn parametry
  *strName = (const char *)cCmd.DataPointer() + sizeof(unsigned);
  return true;
}

bool CClientQuery::ListAlias(const char *pszFrom, std::string *strAlias, std::string *strCity)
{
  // vstupn parametry
  CCmdBuffer cCmdBuffer(CCommand::CMDLISTALIAS, static_cast<unsigned>(strlen(pszFrom) + 1));
  cCmdBuffer.String(pszFrom, 0);
  CCommand cCmd(++m_cCounter & CMD_COUNTER_MASK, cCmdBuffer.Data(), cCmdBuffer.Length());
  cCmdBuffer.Clear();
  // vzdlen voln zpracovatelsk metody
  CallQuery(&cCmd);
  // nvratov hodnota
  bool fResult = *(unsigned *)cCmd.DataPointer() != 0;
  if(!fResult)
    return false;
  // vstupn parametry
  unsigned uiOffset = sizeof(unsigned);
  *strAlias = (const char *)cCmd.DataPointer() + uiOffset;
  uiOffset += static_cast<unsigned>(strlen((const char *)cCmd.DataPointer() + uiOffset) + 1);
  *strCity = (const char *)cCmd.DataPointer() + uiOffset;
  return true;
}

void CClientQuery::CallQuery(CCommand *cCmd)
{
  const char *pszErrorText;
  unsigned uiStatus;
  unsigned uiCounter = cCmd->Status(); // stav tae ped operac
  switch(m_cConnection->Process(cCmd)) { // voln vzdlen metody
    case CMD_SUCCESS:
      // kontrola stavovho slova
      uiStatus = cCmd->Status();
      if((uiStatus & CMD_COUNTER_MASK) != uiCounter)
        pszErrorText = "remote call message mismatch";
      else if(uiStatus & CMD_FUNCFAIL_FLAG)
        pszErrorText = "remote procedure not defined";
      else if(uiStatus & CMD_COMMFAIL_FLAG)
        pszErrorText = "communication error";
      else if(CMD_GETFUNCRESULT(uiStatus) != 0)
        pszErrorText = "remore procedure returned error";
      else
        pszErrorText = NULL;
      break;
    case CMD_ERR_CONNECT: pszErrorText = "server connection error"; break;
    case CMD_ERR_WRITE: pszErrorText = "sending to server failed"; break;
    case CMD_ERR_READ: pszErrorText = "reading from server failed"; break;
    case CMD_ERR_TIMEOUT: pszErrorText = "server timeout expired"; break;
    case CMD_ERR_NOSESSION: pszErrorText = "server not connected"; break;
    default: pszErrorText = "remote call error"; break;
  }
  if(pszErrorText != NULL)
    throw std::runtime_error(pszErrorText);
}
