#ifndef _CLIENTQUERY
#define _CLIENTQUERY

#include <string>
#include <list>
#include "..\Common\Query.h"
#include "..\Common\Command.h"
#include "ClientConn.h"

/*!
@file ClientQuery.h
Implementace tdy @ref CClientQuery, kter na klientsk stran zajiuje sprvn pevod pkaz 
pro databzov rozhran @ref CQuery do binrn podoby vhodn k transportu na server.
*/

//! pevod databzovho dotazu do binrn podoby
/*!
Tda slou jako zstupce databzovho rozhran @ref CQuery na stran klienta. Implementuje vechny
metody zmnnho rozhran. Pi jejich vyvoln vak nekontaktuje databzi pmo, ale pomoc rozhran
@ref CClientConnection je odesl na server ke zpracovn. Pebr zpt nvratov hodnoty a odevzdv
je volajcmu.

Tda zabezpeuje pevod vech voln do pole binrnch dat tdy @ref CCommand, kter je nsledn 
pomoc meziprocesov komunikace odeslno na server. Po obdren odpovdi nastv opan proces - sestaven
nvratovch hodnot z pole binrnch dat. Je zejm, e kdovn/dekdovn dat mus probhat konzistentn
jak na stran klienta, tak na stran serveru, kde tuto innost vykonv sestersk tda @ref CServerQuery.

Tda @ref CCommand krom binrnch dat pen i stavov slovo, kter slou ke kontrole komunikanho 
kanlu. Pokud dojde k njakm problmm (nap. nedostupn server), je takov chybov stav vyhodnocen a 
nadzenmu oznmen vyvolnm pslun vjimky, kter obsahuje i popis piny.

Problematikou meziprocesov komunikace se detailnji zabv strnka @ref ipc.
*/
class CClientQuery : public CQuery {
public:
  //! konstruktor
  /*!
  Vytvoen instance tdy s uloenm ukazatele na rozhran @ref CClientConnection, kter zajiuje
  transport vytvoench zprv (instanc @ref CCommand) na server pomoc meziprocesov komunikace.
  @param[in] cConnection ukazatel na komunikan tdu
  */
  CClientQuery(CClientConnection *cConnection);
  //! destruktor
  virtual ~CClientQuery();
  virtual unsigned AddCity(const char *pszName);
  virtual bool AddCityAlias(const char *pszName, unsigned uiId);
  virtual bool DeleteCity(const char *pszName);
  virtual unsigned FindCity(const char *pszName);
  virtual bool CityInfo(unsigned uiCity, RCityInfo *prInfo, unsigned *puiLinesOut, unsigned *puiLinesIn);
  virtual unsigned AddBusPlan(const char *pszName, unsigned uiSeats, const CBlobEx<std::string> *cData);
  virtual bool DeleteBusPlan(const char *pszName);
  virtual unsigned FindBusPlan(const char *pszName);
  virtual bool BusPlanInfo(unsigned uiPlan, RPlanInfo *prInfo, unsigned *puiBuses, CBlobEx<std::string> *cData);
  virtual unsigned AddBus(const RBusInfo *prBus);
  virtual bool DeleteBus(unsigned uiBus);
  virtual unsigned FindBus(unsigned uiFrom, unsigned uiTo, time_t tLeaveTime);
  virtual bool BusInfo(unsigned uiBus, RBusInfo *prInfo, bool *pfEnabled, unsigned *puiSeats, std::list<unsigned> *cFreeSeats);
  virtual bool EnableBus(unsigned uiBus);
  virtual bool DisableBus(unsigned uiBus);
  virtual unsigned NewCustomer(void);
  virtual bool AskSeats(unsigned uiBus, std::list<unsigned> *cSeats, unsigned uiCustomer);
  virtual bool LockSeats(unsigned uiBus, std::list<unsigned> *cSeats, unsigned uiCustomer);
  virtual bool UnlockSeats(unsigned uiBus, std::list<unsigned> *cSeats, unsigned uiCustomer);
  virtual unsigned NextBus(unsigned uiBus);
  virtual unsigned NextCity(unsigned uiCity);
  virtual unsigned NextPlan(unsigned uiPlan);
  virtual bool ListCity(const char *pszFrom, std::string *strName);
  virtual bool ListAlias(const char *pszFrom, std::string *strAlias, std::string *strCity);
private:
  CClientConnection *m_cConnection;
  unsigned m_cCounter;
  bool SeatOperation(CCommand::EQueryCmd eCmd, unsigned uiBus, std::list<unsigned> *cSeats, unsigned uiCustomer);
  void CallQuery(CCommand *cCmd);
};

#endif
