#define _CRT_SECURE_NO_WARNINGS
#include <cstdio>
#include <string>
#include <time.h>
#include <iostream>
#include <algorithm>
#include <ctype.h>
#include <iomanip>
#include <vector>

#pragma hdrstop

#include "Graphics.h"
#include "Logic.h" // RBusPrint

CGraphics::CGraphics(void)
{
}

CGraphics::~CGraphics()
{
}

bool CGraphics::Read(std::string &strBuffer)
{
  std::cout << "> ";
  getline(std::cin, strBuffer);
  return !std::cin.fail();
}

void CGraphics::Write(const char *pszBuffer)
{
  std::cout << pszBuffer << std::endl;
}

static void WriteBusHeader(void)
{
  std::cout << std::setiosflags(std::ios::left) << std::setw(5) << "ID" << std::setw(5) << "LINE" << std::setw(20) << "FROM" <<
    std::setw(20) << "TO" << std::setw(17) << "TIME" << std::setw(12) << "PLAN" << std::resetiosflags(std::ios::left) << std::endl;
}

static void WriteBusItem(const RBusPrint &rBus)
{
  std::tm *prTime = localtime(&rBus.tLeaveTime);
  std::cout << std::setiosflags(std::ios::left) << std::setw(5) << rBus.uiId << std::setw(5) << rBus.uiLine <<
    std::setw(20) << rBus.strCityFrom << std::setw(20) << rBus.strCityTo << std::setfill('0') <<
    std::resetiosflags(std::ios::left) << std::setw(2) << prTime->tm_hour << ":" << std::setw(2) <<
    prTime->tm_min << "-" << std::setw(2) << prTime->tm_mday << "." << std::setw(2) << prTime->tm_mon+1 <<
    "." << std::setfill(' ') << std::setiosflags(std::ios::left) << std::setw(5) << prTime->tm_year+1900 <<
    std::setw(12) << rBus.strBusPlan << std::endl;
}

void CGraphics::WriteSeats(const char *pszPrefix, std::list<unsigned> *cSeats)
{
  std::cout << pszPrefix;
  std::list<unsigned>::iterator i = cSeats->begin();
  if(i == cSeats->end())
    std::cout << "---";
  else {
    unsigned uiLast = *i;
    std::cout << uiLast;
    bool fRange = false;
    for(++i; i != cSeats->end(); ++i) {
      if(*i == uiLast + 1) {
        uiLast++;
        fRange = true;
      }
      else {
        if(fRange)
          std::cout << "-" << uiLast;
        uiLast = *i;
        std::cout << " " << uiLast;
        fRange = false;
      }
    }
    if(fRange)
      std::cout << "-" << uiLast;
    std::cout << " (total: " << cSeats->size() << ")";
  }
  std::cout << std::endl;
}

void CGraphics::WriteBuses(std::list<RBusPrint> *cBuses)
{
  WriteBusHeader();
  for_each(cBuses->begin(), cBuses->end(), [](const RBusPrint &rBus){WriteBusItem(rBus);});
}

void CGraphics::WriteBusPlan(std::string &strPlan, std::list<unsigned> *cFreeSeats, RBusPrint *cBusData,
  unsigned uiNumberSeats, const char* pszPrefix)
{
  WriteBusHeader();
  WriteBusItem(*cBusData);
  Write("Bus plan:");
  auto j = cFreeSeats->begin();
  for(unsigned i = 1; i <= uiNumberSeats; i++) {
    if(j == cFreeSeats->end() || *j > i) {
      std::string strNumber = " " + std::to_string(i) + " ";
      unsigned uiLength = static_cast<unsigned>(strNumber.length() - 2);
      unsigned uiPos = static_cast<unsigned>(strPlan.find(strNumber));
      std::string sNew = " ";
      for(unsigned i = 0; i < uiLength; i++)
        sNew += "-";
      sNew += " ";
      strPlan.replace(uiPos, uiLength + 2, sNew);
    }
    else if(*j < i) {
      while(*j <= i)
          ++j;
    }
    else
      ++j;
  }
  Write(strPlan.c_str());
  WriteSeats(pszPrefix, cFreeSeats);
}

bool CGraphics::Continue(void)
{
  std::cout << "Write \'y\' to continue: ";
  char cChar;
  std::cin.get(cChar);
  if(cChar != '\n') {
    while(std::cin.get() != '\n')
     ;
  }
  return tolower(cChar) == 'y';
}

void CGraphics::WriteReservation(unsigned uiId, std::list<unsigned> *cSeats, RBusPrint *prBusPrint)
{
  std::tm *prTime;

  prTime = localtime(&prBusPrint->tLeaveTime);
  std::cout << std::endl << "Your ticket:" << std::endl << std::endl;
  std::cout << std::setiosflags(std::ios::left) << std::setw(9) << "USER ID" << std::setw(8) << "BUS ID" << std::setw(20) << "CITY FROM" <<
    std::setw(20) << "CITY TO" << std::setw(20) << "LEAVE TIME" << std::resetiosflags(std::ios::left) << std::endl;
  std::cout << std::setiosflags(std::ios::left) << std::setw(9) << uiId << std::setw(8) << prBusPrint->uiId <<
    std::setw(20) << prBusPrint->strCityFrom << std::setw(20) << prBusPrint->strCityTo << std::setfill('0') <<
    std::resetiosflags(std::ios::left) << std::setw(2) << prTime->tm_hour << ":" << std::setw(2) <<
    prTime->tm_min << "-" << std::setw(2) << prTime->tm_mday << "." << std::setw(2) << prTime->tm_mon+1 <<
    "." << std::setfill(' ') << std::setiosflags(std::ios::left) << std::setw(8) << prTime->tm_year+1900 << std::endl;
  WriteSeats("Reserved seats: ", cSeats);
}

void CGraphics::WriteAddCity(unsigned uiId)
{
  std::cout << "OK. City number: " << uiId << std::endl;
}

void CGraphics::WriteAddBus(unsigned uiId)
{
  std::cout << "OK. Bus number: " << uiId << std::endl;
}

void CGraphics::WriteAddPlan(unsigned uiId)
{
  std::cout << "OK. Plan number: " << uiId << std::endl;
}

void CGraphics::WriteDeleted(unsigned uiCount)
{
  if(uiCount == 0)
    std::cout << "OK. No item deleted." << std::endl;
  else
    std::cout << "OK. Items deleted: " << uiCount << std::endl;
}

void CGraphics::WriteHelp(bool fFullClient)
{
  std::cout << "List of available commands:" << std::endl;
  std::cout << "EXIT\tcloses the program" << std::endl;
  std::cout << "FIND\tfinds 3 buses\n\tExamples:\tfind Sobeslav Praha 12:20\n\t\t\tfind \"Hradec Kralove\" Praha 15:18-2.10.2013" << std::endl;
  std::cout << "NEXT\tfinds next 3 buses - use FIND first" << std::endl;
  std::cout << "BUS\tsets active bus by number\n\tExample: bus 15" << std::endl;
  std::cout << "RESERVE\treserves seats in current bus\n\tExamples:\treserve 4,5,6\t- reserves seats 4, 5, 6\n\t\t\treserve #4\t- reserves 4 first seats" << std::endl;
  std::cout << "UNLOCK\tunlocks reserved seats in current bus\n\tExamples:\tunlock 2\t- unlocks seats with customer number 2\n\t\t\tunlock 2 4,5,6\t- unlock seats 4, 5, 6 with customer number 2" << std::endl;
  if(fFullClient) {
    std::cout << "ADDBUS\tadds new bus\n\tExample:\taddbus 12 Sobeslav Praha 15:18-2.10.2013 Karosa652\n\t\t- adds bus from Sobeslav to Praha at 15:18-2.10.2013\n\t\t  with line number 12 and bus plan Karosa652" << std::endl;
    std::cout << "DELBUS\tdeletes bus with specified number\n\tExample: delbus 5" << std::endl;
    std::cout << "ADDCITY\tadds new city\n\tExample: addcity Kolin" << std::endl;
    std::cout << "DELCITY\tdeletes city\n\tExample: delcity Kolin" << std::endl;
    std::cout << "ADDCITYALIAS\tadds new alias to existing city\n\tExample: addcityalias HK \"Hradec Kralove\"" << std::endl;
    std::cout << "ADDBUSPLAN\tadds new bus plan\n\tExample: addbusplan Karosa652 42 karosa652.txt\n\t\t- adds bus plan with name Karosa652 with 42 seats\n\t\t  and text sketch of plan is in file karosa652.txt" << std::endl;
    std::cout << "DELBUSPLAN\tdeletes bus plan\n\tExample: delbusplan Karosa652" << std::endl;
    std::cout << "CLEAR\tclears old buses/unused cities/unused bus plans\n\tExamples:\tclear bus\n\t\t\tclear city\n\t\t\tclear plan" << std::endl;
    std::cout << "CLOSE\tcloses bus for reservations\n\tExample:\tclose 5\t\t- closes bus ID 5" << std::endl;
  }
  std::cout << "CITY\tprints existing cities, optional with given prefix\n\tExample:\tcity\t\t- prints all cities and aliases\n\t\t\tcity Pr\t\t- prints cities and aliases with prefix \'Pr\'" << std::endl;
  std::cout << "HELP\tprints this help page" << std::endl;
  std::cout << "\nCommands are case insensitive." << std::endl;
}

void CGraphics::WriteWelcome(void)
{
  std::cout << "Bus Reservation System 1.0\nPetr Stefan - Programming in C++, 2013/2014" << std::endl;
}

void CGraphics::WriteCities(std::vector<std::pair<std::string, std::string>>& cOut)
{
  for(auto it = cOut.begin(); it != cOut.end(); ++it) {
    if(it->second.empty()) {
      std::cout << it->first << std::endl;
    }
    else {
      std::cout << it->second << " -> " << it->first << std::endl;
    }
  }
}
