#ifndef _GRAPHICS_H
#define _GRAPHICS_H

#include <list>
#include <vector>
#include "Logic.h"

/*!
@file Graphics.h
Tda zajiujc vstup a vstup.
*/

//! grafick interface
/*!
Tda slou k zskvn vstupu od uivatele a sdlovn zptky vsledek. Vechny informace jsou sdlovny pes
standartn vstup a vstup.
*/
class CGraphics {
public:
  //! konstruktor
  CGraphics(void);
  //!destruktor
  ~CGraphics();
  //! peten dky ze vstupu
  /*!
  Metoda pete ze vstupu dek a ulo ho do pipravenho bufferu. Z dky odstran
  ppadn znak newline '\\n'.
  @param[out] strBuffer buffer k naplnn etzcem zadanm uivatelem
  @return @e true pi spchu, @e false jinak (nap. pi dosaen konce souboru v ppad, kdy je soubor
  pesmrovn na standardn vstup)
  */
  bool Read(std::string &strBuffer);
  //! zapsn C-kovho etzce na vstup
  /*!
  @param[in] pszBuffer etzec ukonen '\\0', kter bude vypsn na standardn vstup
  */
  void Write(const char *pszBuffer);
  //! vypsn autobus
  /*!
  Vypsn autobus uloench v kontejneru @e list, pouv se pro vpis po pkazech "find" a "next".
  @param[in] cBuses list struktur @ref RBusPrint obsahujc informace o vypisovanch autobusech
  */
  void WriteBuses(std::list<RBusPrint> *cBuses);
  //! vypsn sedadel
  /*!
  Vypsn sedadel v iteln form (pokud je to mon, tak rozsahem - nap. msto <tt>5 6 7 8 11 12</tt>
  je uvedeno <tt>5-8 11-12</tt>) spolu s jejich potem.
  @param[in] pszPrefix text uveden ped seznamem sedadel
  @param[in] cSeats seznam sedadel (pedpokld se, e seznam je vzestupn setdn a neobsahuje
  duplicitn zznamy)
  */
  void WriteSeats(const char *pszPrefix, std::list<unsigned> *cSeats);
  //! vypsn plnu autobusu
  /*!
  @param[in] strPlan textov podoba plnu autobusu
  @param[in] cFreeSeats seznam volnch sedadel
  @param[in] cBusData informace o autobusu
  @param[in] uiNumberSeats celkov poet sedadel autobusu
  @param[in] pszPrefix etzec vytitn ped vpis sedadel
  */
  void WriteBusPlan(std::string &strPlan, std::list<unsigned> *cFreeSeats, RBusPrint *cBusData, unsigned uiNumberSeats, const char* pszPrefix = "Free seats: ");
  //! vypsn jzdenky po spn rezervaci
  /*!
  Po spn rezervace sedadel je uivateli vystaveno potvrzen se vemi potebnmi daji
  (vetn sla zakaznka). Jejich pomoc me nsledn provst i zruen rezervace.
  @param[in] uiId ID zkaznka
  @param[in] cSeats seznam zarezervovanch sedadel
  @param[in] prBusPrint informace o autobusu
  */
  void WriteReservation(unsigned uiId, std::list<unsigned> *cSeats, RBusPrint *prBusPrint);
  //! vypsn ID nov pidanho msta
  /*!
  @param[in] uiId ID nov pidanho msta
  */
  void WriteAddCity(unsigned uiId);
  //! vypsn ID nov pidanho autobusu
  /*!
  @param[in] uiId ID nov pidanho autobusu
  */
  void WriteAddBus(unsigned uiId);
  //! vypsn ID nov pidanho plnu autobusu
  /*!
  @param[in] uiId ID nov pidanho plnu autobusu
  */
  void WriteAddPlan(unsigned uiId);
  //! potvrzen volby
  /*!
  Metoda vype vzvu k potvrzen, nsledn ek na odpov (vstup zstane po konci funkce przdn).
  @return @e true v ppad potvrzen uivatelem, jinak @e false
  */
  bool Continue(void);
  //! vypsn npovdy
  void WriteHelp(bool fFullClient);
  //! vypsn hlaviky pi sputn
  void WriteWelcome(void);
  //! vypsn potu smazanch poloek
  /*!
  @param[in] uiCount poet smazanch poloek
  */
  void WriteDeleted(unsigned uiCount);
  //! vypasn existujch mst
  /*!
  @param[in] cOut seznam mst k vypsn. Nzev msta je v prvnm stringu. Pokud nalezen
  vsledek je alias, je uloen v druhm stringu (pokud je druh string neprzdn, vypisuje
  se alias a za nm msto)
  */
  void WriteCities(std::vector<std::pair<std::string, std::string>>& cOut);
};

#endif
