#define _CRT_SECURE_NO_WARNINGS

#include <string>
#include <time.h>
#include <list>
#include <iostream>
#include <fstream>
#include <ctime>
#include <regex>

#pragma hdrstop

#include "Graphics.h"
#include "../Common/Query.h"
#include "Logic.h"


CLogic::CLogic(CQuery *cQuery) : m_tCurLeaveTime(0), m_uiCurBus(0), m_uiCurCityFrom(0), m_uiCurCityTo(0),
  m_uiCurUserId(0), m_tNextLeaveTime(0), m_eLastError(NOERR), m_cDatabase(cQuery)
{
}

CLogic::~CLogic()
{
}

void CLogic::Init(void)
{
  Clear();
  m_uiCurUserId = m_cDatabase->NewCustomer();
}

void CLogic::Clear(void)
{
  m_eLastError = NOERR;
  m_tCurLeaveTime = 0;
  m_uiCurBus = 0;
  m_uiCurCityFrom = 0;
  m_uiCurCityTo = 0;
  m_uiCurUserId = 0;
}

bool CLogic::Find(std::list<RBusPrint> *cBuses, const char *pszCityFrom, const char *pszCityTo, const char *pszLeaveTime)
{
  m_eLastError = NOERR;
  m_uiCurCityFrom = m_cDatabase->FindCity(pszCityFrom);
  if(m_uiCurCityFrom == 0) {
    m_eLastError = CITYFROM;
    return false;
  }
  m_uiCurCityTo = m_cDatabase->FindCity(pszCityTo);
  if(m_uiCurCityTo == 0) {
    m_eLastError = CITYTO;
    return false;
  }
  m_tCurLeaveTime = ConvertTime(pszLeaveTime);
  if(m_tCurLeaveTime == 0) {
    m_eLastError = LEAVETIME;
    return false;
  }
  m_tNextLeaveTime = m_tCurLeaveTime;
  if(Next(cBuses))
    return true;
  else {
    m_eLastError = NBUSESFOUND;
    return false;
  }
}

bool CLogic::Next(std::list<RBusPrint> *cBuses)
{
  unsigned uiCurBus;
  RBusInfo rBusInfo;
  RBusPrint rBusPrint;
  RCityInfo rCityInfo;
  RPlanInfo rPlanInfo;
  
  m_eLastError = NOERR;
  if(m_tNextLeaveTime == 0 || m_uiCurCityFrom == 0 || m_uiCurCityTo == 0) {
    m_eLastError = FIRSTFIND;
    return false;
  }
  for(unsigned i = 0; i < uiPrintCount; i++) {
    uiCurBus = m_cDatabase->FindBus(m_uiCurCityFrom, m_uiCurCityTo, m_tNextLeaveTime);
    if(uiCurBus != 0) {
      m_cDatabase->BusInfo(uiCurBus, &rBusInfo, NULL, NULL, NULL);
      rBusPrint.uiId = rBusInfo.uiId;
      rBusPrint.tLeaveTime = rBusInfo.tLeaveTime;
      rBusPrint.uiLine = rBusInfo.uiLine;
      m_cDatabase->CityInfo(rBusInfo.uiCityFrom, &rCityInfo, NULL, NULL);
      rBusPrint.strCityFrom = rCityInfo.strName;
      m_cDatabase->CityInfo(rBusInfo.uiCityTo, &rCityInfo, NULL, NULL);
      rBusPrint.strCityTo = rCityInfo.strName;
      m_cDatabase->BusPlanInfo(rBusInfo.uiPlan, &rPlanInfo, NULL, NULL);
      rBusPrint.strBusPlan = rPlanInfo.strName;
      cBuses->push_back(rBusPrint);
      m_tNextLeaveTime = rBusInfo.tLeaveTime + 1;
      if(i == 0)
        m_tCurLeaveTime = rBusInfo.tLeaveTime;
    }
    else {
      if(cBuses->empty()) {
        m_eLastError = NBUSESFOUND;
        return false;
      }
      break;
    }
  }
  return true;
}

bool CLogic::SetBus(unsigned uiBusId, std::list<unsigned> *cFreeSeats, std::string *strPlan, RBusPrint *prBus, unsigned *puiNumberSeats)
{
  RBusInfo rBusInfo;
  RCityInfo rCityInfo;
  RPlanInfo rPlanInfo;
  CBlobEx<std::string> cBlob;
  bool fEnable;
  unsigned uiNumberSeats;
  m_eLastError = NOERR;
  if(m_cDatabase->BusInfo(uiBusId, &rBusInfo, &fEnable, &uiNumberSeats, cFreeSeats)) {
    if(puiNumberSeats != NULL)
      *puiNumberSeats = uiNumberSeats;
    if(!fEnable) {
      m_eLastError = BUSDISABLED;
      return false;
    }
    m_uiCurBus = uiBusId;
    m_cDatabase->BusPlanInfo(rBusInfo.uiPlan, &rPlanInfo, NULL, &cBlob);
    *strPlan = cBlob.Get();
    if(prBus != NULL) {
      prBus->uiId = m_uiCurBus;
      prBus->uiLine = rBusInfo.uiLine;
      m_cDatabase->CityInfo(rBusInfo.uiCityFrom, &rCityInfo, NULL, NULL);
      prBus->strCityFrom = rCityInfo.strName;
      m_cDatabase->CityInfo(rBusInfo.uiCityTo, &rCityInfo, NULL, NULL);
      prBus->strCityTo = rCityInfo.strName;
      prBus->tLeaveTime = rBusInfo.tLeaveTime;
      prBus->strBusPlan = rPlanInfo.strName;
    }
    return true;
  }
  else {
    m_eLastError = WRONGBID;
    return false;
  }
}

// list je jen vstupn parametr, zamluv se poet sedadel uiSeatsCount
bool CLogic::ReserveAsk(std::list<unsigned> *cSeats, unsigned uiSeatsCount)
{
  m_eLastError = NOERR;
  if(m_uiCurBus == 0) {
    m_eLastError = NBUSSET;
    return false;
  }
  if(m_uiCurUserId == 0) {
    m_eLastError = NUSERSET;
    return false;
  }
  if(uiSeatsCount == 0) {
    m_eLastError = WRSEATSCOUNT;
    return false;
  }
  do {
    m_cDatabase->BusInfo(m_uiCurBus, NULL, NULL, NULL, cSeats);
    unsigned uiSeats = static_cast<unsigned>(cSeats->size());
    if(uiSeats < uiSeatsCount) {
      cSeats->clear();
      m_eLastError = NFREESEATS;
      return false; // neni tolik volnch mst
    }
    else {
      uiSeats -= uiSeatsCount;
      while(uiSeats-- != 0)
        cSeats->pop_back();
    }
  } while(!m_cDatabase->AskSeats(m_uiCurBus, cSeats, m_uiCurUserId));
  return true;
}

bool CLogic::ReserveAsk(std::list<unsigned> *cSeats)
{
  m_eLastError = NOERR;
  if(m_uiCurBus == 0) {
    m_eLastError = NBUSSET;
    return false;
  }
  if(m_uiCurUserId == 0) {
    m_eLastError = NUSERSET;
    return false;
  }
  if(cSeats->empty()) {
    m_eLastError = WRITESEATS;
    return false;
  }
  if(!m_cDatabase->AskSeats(m_uiCurBus, cSeats, m_uiCurUserId)) {
    m_eLastError = NRESERVE;
    return false;
  }
  else
    return true;
}

bool CLogic::ReserveForever(std::list<unsigned> *cSeats, unsigned *puiId, RBusPrint *prBus)
{
  RBusInfo rBusInfo;
  RCityInfo rCityInfo;

  m_eLastError = NOERR;
  if(m_uiCurBus == 0 || m_uiCurUserId == 0)
    return false;
  if(puiId != NULL)
    *puiId = m_uiCurUserId;
  if(prBus != NULL) {
    prBus->uiId = m_uiCurBus;
    m_cDatabase->BusInfo(m_uiCurBus, &rBusInfo, NULL, NULL, NULL);
    prBus->uiLine = rBusInfo.uiLine;
    m_cDatabase->CityInfo(rBusInfo.uiCityFrom, &rCityInfo, NULL, NULL);
    prBus->strCityFrom = rCityInfo.strName;
    m_cDatabase->CityInfo(rBusInfo.uiCityTo, &rCityInfo, NULL, NULL);
    prBus->strCityTo = rCityInfo.strName;
    prBus->tLeaveTime = rBusInfo.tLeaveTime;
  }
  bool fResult = m_cDatabase->LockSeats(m_uiCurBus, cSeats, m_uiCurUserId);
  m_uiCurUserId = m_cDatabase->NewCustomer(); // nov slo zkaznka
  return fResult;
}

bool CLogic::UnlockAll(unsigned uiUserId, std::list<unsigned> *cSeats)
{
  m_eLastError = NOERR;
  if(uiUserId == 0) {
    m_eLastError = WRUSERID;
    return false;
  }
  if(m_uiCurBus == 0) {
    m_eLastError = NBUSSET;
    return false;
  }
  ReservedSeats(m_uiCurBus, cSeats);
  m_cDatabase->UnlockSeats(m_uiCurBus, cSeats, uiUserId);
  return true;
}

bool CLogic::Unlock(std::list<unsigned> *cSeats)
{
  m_eLastError = NOERR;
  return Unlock(m_uiCurUserId, cSeats);
}

bool CLogic::Unlock(unsigned uiUserId, std::list<unsigned> *cSeats)
{
  m_eLastError = NOERR;
  if(uiUserId == 0) {
    m_eLastError = WRUSERID;
    return false;
  }
  if(m_uiCurBus == 0) {
    m_eLastError = NBUSSET;
    return false;
  }
  if(m_cDatabase->UnlockSeats(m_uiCurBus, cSeats, uiUserId))
    return true;
  else {
    m_eLastError = NUNLOCK;
    return false;
  }
}

unsigned CLogic::AddCity(const char *pszCityName)
{
  m_eLastError = NOERR;
  if(*pszCityName == '\0') {
    m_eLastError = WRCITY;
    return 0;
  }
  return m_cDatabase->AddCity(pszCityName);
}

bool CLogic::DelCity(const char *pszCityName)
{
  m_eLastError = NOERR;
  if(*pszCityName == '\0') {
    m_eLastError = WRCITY;
    return 0;
  }
  if(m_cDatabase->DeleteCity(pszCityName))
    return true;
  else {
    m_eLastError = NDELCITY;
    return false;
  }
}

bool CLogic::AddCityAlias(const char *pszCityAlias, const char *pszCityName)
{
  m_eLastError = NOERR;
  if(*pszCityAlias == '\0') {
    m_eLastError = WRALIAS;
    return false;
  }
  if(*pszCityName == '\0') {
    m_eLastError = WRCITY;
    return false;
  }
  unsigned uiId = m_cDatabase->FindCity(pszCityName);
  if(uiId == 0) {
    m_eLastError = CTNFOUND;
    return false;
  }
  return m_cDatabase->AddCityAlias(pszCityAlias, uiId);
}

unsigned CLogic::AddBus(unsigned uiLine, const char *pszCityFrom, const char *pszCityTo, const char *pszLeaveTime, const char *pszPlan)
{
  RBusInfo rBus;
  
  m_eLastError = NOERR;
  if(uiLine == 0) {
    m_eLastError = WRITEBLINE;
    return 0;
  }
  if(*pszCityFrom == '\0') {
    m_eLastError = CITYFROM;
    return 0;
  }
  if(*pszCityTo == '\0') {
    m_eLastError = CITYTO;
    return 0;
  }
  if(*pszLeaveTime == '\0') {
    m_eLastError = LEAVETIME;
    return 0;
  }
  if(*pszPlan == '\0') {
    m_eLastError = WRBUstrPlan;
    return 0;
  }
  unsigned uiCityFrom = m_cDatabase->FindCity(pszCityFrom);
  if(uiCityFrom == 0) {
    m_eLastError = CTFROMNFOUND;
    return 0;
  }
  unsigned uiCityTo = m_cDatabase->FindCity(pszCityTo);
  if(uiCityTo == 0) {
    m_eLastError = CTTONFOUND;
    return 0;
  }
  unsigned uiPlan = m_cDatabase->FindBusPlan(pszPlan);
  if(uiPlan == 0) {
    m_eLastError = PLANNFOUND;
    return 0;
  }
  time_t tLeaveTime = ConvertTime(pszLeaveTime);
  if(tLeaveTime == 0) {
    m_eLastError = TIMEFORMAT;
    return 0;
  }
  rBus.tLeaveTime = tLeaveTime;
  rBus.uiCityFrom = uiCityFrom;
  rBus.uiCityTo = uiCityTo;
  rBus.uiPlan = uiPlan;
  rBus.uiLine = uiLine;
  unsigned uiBus = m_cDatabase->AddBus(&rBus);
  if(uiBus == 0)
    m_eLastError = NADDBUS;
  return uiBus;
}

bool CLogic::DelBus(unsigned uiBusId)
{
  m_eLastError = NOERR;
  if(uiBusId == 0) {
    m_eLastError = WRITEBID;
    return false;
  }
  bool fEnabled;
  RBusInfo rBus;
  m_cDatabase->BusInfo(uiBusId, &rBus, &fEnabled, NULL, NULL);
  if(fEnabled && rBus.tLeaveTime > time(NULL)) {
    m_eLastError = USEDBUS;
    return false;
  }
  if(m_cDatabase->DeleteBus(uiBusId))
    return true;
  else {
    m_eLastError = NDELBUS;
    return false;
  }
}

unsigned CLogic::AddBusPlan(const char *pszName, unsigned uiSeats, const char *pszFile)
{
  m_eLastError = NOERR;
  if(*pszName == '\0') {
    m_eLastError = WRITEBPLAN;
    return 0;
  }
  if(uiSeats == 0) {
    m_eLastError = WRSEATSCOUNT;
    return 0;
  }
  if(*pszFile == '\0') {
    m_eLastError = NFILE;
    return 0;
  }
  std::ifstream cInput(pszFile);
  if(cInput.is_open()) {
    std::string strFile;
    std::string strLine;
    while(getline(cInput, strLine)) {
      strFile += strLine;
      strFile += "\n";
    }
    cInput.close();
    if(cInput.fail() && !cInput.eof()) {
      m_eLastError = FILEREADFAIL;
      return 0;
    }
    CBlobEx<std::string> cData(strFile, NULL, 0);
    unsigned uiPlan = m_cDatabase->AddBusPlan(pszName, uiSeats, &cData);
    if(uiPlan == 0)
      m_eLastError = NADDPLAN;
    return uiPlan;
  }
  else {
    m_eLastError = NFILE;
    return 0;
  }
}

bool CLogic::DelBusPlan(const char *pszName)
{
  m_eLastError = NOERR;
  if(*pszName == '\0') {
    m_eLastError = WRBUstrPlan;
    return false;
  }
  if(m_cDatabase->DeleteBusPlan(pszName)) {
    return true;
  }
  else {
    m_eLastError = NDELPLAN;
    return false;
  }
}

unsigned CLogic::ClearBus(void)
{
  unsigned uiCounter = 0;
  unsigned uiBus = 0;
  time_t tNow = time(NULL);
  RBusInfo rInfo;
  while((uiBus = m_cDatabase->NextBus(uiBus)) != 0) {
    m_cDatabase->BusInfo(uiBus, &rInfo, NULL, NULL, NULL);
    if(rInfo.tLeaveTime < tNow)
      if(m_cDatabase->DeleteBus(uiBus))
        uiCounter++;
  }
  m_eLastError = NOERR;
  return uiCounter;
}

unsigned CLogic::ClearCity(void)
{
  unsigned uiCounter = 0;
  unsigned uiOut, uiIn;
  RCityInfo rInfo;
  unsigned uiCity = 0;
  while((uiCity = m_cDatabase->NextCity(uiCity)) != 0) {
    m_cDatabase->CityInfo(uiCity, &rInfo, &uiOut, &uiIn);
    if(uiOut == 0 && uiIn == 0)
      if(m_cDatabase->DeleteCity(rInfo.strName.c_str()))
        uiCounter++;
  }
  m_eLastError = NOERR;
  return uiCounter;
}

unsigned CLogic::ClearPlan(void)
{
  unsigned uiCounter = 0;
  unsigned uiBuses;
  RPlanInfo rInfo;
  unsigned uiPlan = 0;
  while((uiPlan = m_cDatabase->NextPlan(uiPlan)) != 0) {
    m_cDatabase->BusPlanInfo(uiPlan, &rInfo, &uiBuses, NULL);
    if(uiBuses == 0)
      if(m_cDatabase->DeleteBusPlan(rInfo.strName.c_str()))
        uiCounter++;
  }
  m_eLastError = NOERR;
  return uiCounter;
}

bool CLogic::Close(unsigned uiBusId, std::list<unsigned> *cReservedSeats, std::string *strPlan, RBusPrint *prBusPrint, unsigned *puiSeats)
{
  m_eLastError = NOERR;
  if(!m_cDatabase->DisableBus(uiBusId)) {
    m_eLastError = NDISBUS;
    return false;
  }
  ReservedSeats(uiBusId, cReservedSeats, puiSeats);
  RBusInfo rBusInfo;
  RCityInfo rCityInfo;
  RPlanInfo rPlanInfo;
  CBlobEx<std::string> cBlob;
  bool fEnable;
  unsigned uiNumberSeats;
  m_eLastError = NOERR;
  if(m_cDatabase->BusInfo(uiBusId, &rBusInfo, &fEnable, &uiNumberSeats, NULL)) {
    m_uiCurBus = uiBusId;
    m_cDatabase->BusPlanInfo(rBusInfo.uiPlan, &rPlanInfo, NULL, &cBlob);
    *strPlan = cBlob.Get();
    if(prBusPrint != NULL) {
      prBusPrint->uiId = m_uiCurBus;
      prBusPrint->uiLine = rBusInfo.uiLine;
      m_cDatabase->CityInfo(rBusInfo.uiCityFrom, &rCityInfo, NULL, NULL);
      prBusPrint->strCityFrom = rCityInfo.strName;
      m_cDatabase->CityInfo(rBusInfo.uiCityTo, &rCityInfo, NULL, NULL);
      prBusPrint->strCityTo = rCityInfo.strName;
      prBusPrint->tLeaveTime = rBusInfo.tLeaveTime;
      prBusPrint->strBusPlan = rPlanInfo.strName;
    }
    return true;
  }
  return true;
}

void CLogic::City(std::string& in, std::vector<std::pair<std::string, std::string>>& cOut)
{
  m_eLastError = NOERR;
  std::string temp = in;
  std::string answer;
  std::string alias;
  bool fRet;
  if(in.empty()) {
    fRet = m_cDatabase->ListCity(temp.c_str(), &answer);
    while(fRet) {
      cOut.push_back(std::make_pair(answer, alias));
      temp = answer + " ";
      fRet = m_cDatabase->ListCity(temp.c_str(), &answer);
    }
    temp = in;
    fRet = m_cDatabase->ListAlias(temp.c_str(), &alias, &answer);
    while(fRet) {
      cOut.push_back(std::make_pair(answer, alias));
      temp = alias + " ";
      fRet = m_cDatabase->ListAlias(temp.c_str(), &alias, &answer);
    }
  }
  else {
    fRet = m_cDatabase->ListCity(temp.c_str(), &answer);
    while(answer.substr(0, in.size()) == in && fRet) {
      cOut.push_back(std::make_pair(answer, alias));
      temp = answer + " ";
      fRet = m_cDatabase->ListCity(temp.c_str(), &answer);
    }
    temp = in;
    fRet = m_cDatabase->ListAlias(temp.c_str(), &alias, &answer);
    while(alias.substr(0, in.size()) == in && fRet) {
      cOut.push_back(std::make_pair(answer, alias));
      temp = alias + " ";
      fRet = m_cDatabase->ListAlias(temp.c_str(), &alias, &answer);
    }
  }
}

Error CLogic::GetLastError(void) const
{
  return m_eLastError;
}

// zjist seznam volnch sedadel pro zadan autobus a vygeneruje seznam rezervovanch sedadel
bool CLogic::ReservedSeats(unsigned uiBus, std::list<unsigned> *cSeats, unsigned *puiSeats)
{
  unsigned uiSeat;
  std::list<unsigned> cFreeSeats;

  if(cSeats != NULL && m_cDatabase->BusInfo(uiBus, NULL, NULL, &uiSeat, &cFreeSeats)) {
    cSeats->clear();
    std::list<unsigned>::iterator it = cFreeSeats.begin();
    for(unsigned i = 1; i <= uiSeat; i++) {
      if(it != cFreeSeats.end()) {
        if(i < *it)
          cSeats->push_back(i);
        else if (i == *it)
          ++it;
      }
      else
        cSeats->push_back(i);
    }
    if(puiSeats != NULL)
      *puiSeats = uiSeat;
    return true;
  }
  return false;
}

time_t CLogic::ConvertTime(const char *pszTime)
{
  //formt je 13:15-2.10.2013 nebo jen 13:15
  std::tm tmTime;
  std::tm *tmNow;
  time_t tNow;

  tNow = time(NULL);
  tmNow = localtime(&tNow);
  tmTime.tm_isdst = -1;
  tmTime.tm_mday = tmNow->tm_mday;
  tmTime.tm_mon = tmNow->tm_mon;
  tmTime.tm_year = tmNow->tm_year;
  tmTime.tm_hour = 0;
  tmTime.tm_min = 0;
  tmTime.tm_sec = 0;

  std::string strTime(pszTime);
  std::tr1::cmatch cResult;
  std::regex CRegExp("^([0-9][0-9]?):([0-9][0-9]?)(-([0-9][0-9]?)\\.([0-9][0-9]?)\\.([0-9]{4}))?$");
  std::tr1::regex_search(strTime.c_str(), cResult, CRegExp);

  if(cResult.empty())
    return 0;
  int x;
  x = atoi(cResult[1].str().c_str());
  if(cResult[1].matched && x >= 0 && x <= 23)
    tmTime.tm_hour = x;
  else
    return 0;
  x = atoi(cResult[2].str().c_str());
  if(cResult[2].matched && x >= 0 && x <= 59)
    tmTime.tm_min = x;
  else
    return 0;
  x = atoi(cResult[4].str().c_str());
  if(cResult[4].matched) {
    if(x >= 1 && x <= 31)
      tmTime.tm_mday = x;
    else
      return 0;
  }
  x = atoi(cResult[5].str().c_str());
  if(cResult[5].matched) {
    if(x >= 1 && x <= 12)
      tmTime.tm_mon = --x;
    else
      return 0;
  }
  x = atoi(cResult[6].str().c_str());
  if(cResult[6].matched) {
    if(x >= 1970 && x <= 3000)
      tmTime.tm_year = x - 1900;
    else
      return 0;
  }
  return mktime(&tmTime);
}