#include <winsock2.h>
#include <string>
#include <iostream>
#include <fstream>
#include <crtdbg.h>

#pragma hdrstop

#include "ClientApp.h"
#include "ClientConn.h"
#include "ClientMsg.h"
#include "ClientTcp.h"
#include "ClientQuery.h"


bool ReadConf(std::string& strConf, std::string* strTransport, std::string* strEndpoint, bool* fFullClient)
{
  bool fReturn = true;
  *fFullClient = false;
  std::ifstream cInput(strConf);
  if(cInput.is_open()) {
    std::string strLine;
    while(getline(cInput, strLine)) {
      if(strLine[0] == '#')
        continue;
      if(strLine.find("transport") == 0) {
        *strTransport = strLine.substr(strLine.find("=") + 1);
      }
      else if(strLine.find("endpoint") == 0) {
        *strEndpoint = strLine.substr(strLine.find("=") + 1);
      }
      else if(strLine.find("full_client") == 0) {
        if(strLine.substr(strLine.find("=") + 1) == "1")
          *fFullClient = true;
      }
      else {
        std::cout << "Wrong format of brs_client.conf file." << std::endl;
        fReturn = false;
      }
    }
    cInput.close();
    if(cInput.fail() && !cInput.eof()) {
      std::cout << "Error reading brs_client.conf file." << std::endl;
      fReturn = false;
    }
  }
  else {
    std::cout << "Cannot open brs_client.conf file." << std::endl;
    fReturn = false;
  }
  return fReturn;
}



int main(int argc, char **argv)
{
#ifdef _DEBUG
	_CrtSetDbgFlag( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );
#endif


  std::string strPath(argv[0]);
  std::string strConf = strPath.substr(0, strPath.find_last_of("/\\") + 1);
  strConf += "brs_client.conf";

  std::string strTransport;
  std::string strEndpoint;
  bool fFullClient;
  if(!ReadConf(strConf, &strTransport, &strEndpoint, &fFullClient)) {
    char cChar;
    std::cin.get(cChar);
    return 0;
  }

  CClientConnection *cClient;
  if(strTransport == "tcp")
    cClient = new CClientTcp(strEndpoint.c_str());
  else if(strTransport == "msg")
    cClient = new CClientMsg(strEndpoint.c_str());
  else {
    std::cout << "Wrong 'transport' parameter in brs_client.conf file." << std::endl;
    char cChar;
    std::cin.get(cChar);
    return 0;
  }
  std::cout << "Client type: " << strTransport << "    Client endpoint: " << strEndpoint << std::endl << std::endl;
  CQuery *cClientQuery = new CClientQuery(cClient);
  CClientApp cTerminal(cClientQuery, fFullClient);
  cTerminal.Run();
  delete cClient;
  delete cClientQuery;
  return 0;
}
