#ifndef _COMMAND_H
#define _COMMAND_H

#include <windows.h>
#include "UtilClass.h"

/*!
@file Command.h
Deklarace tdy @ref CCommand a dalch prostedk pro manipulaci s stmi stavovho slova.
@anchor cmd_utils
*/
//! tda pro zapouzden pedvanch dat na klientu
/*! 
Tda reprezentuje data na klientsk stran, slou jako parametr metod pro odesln i pjem pkaz.
Data intern sestvaj z nsledujcch st:
@li pole byt, kter odpovd binrn podob dat,
@li velikost dat (tj. dlka pole),
@li stavov slovo slouc k penosu informac o spchu komunikace a zpracovn dat.

<b>Stavov slovo</b> pkazu obsahuje informace o vsledku komunikace klienta a serveru a zpracovn pkazu 
na stran serveru. Jedn se o celoseln typ, kter se skld z nkolika st:
@li ta - hodnota pro libovoln pouit na stran klienta,
@li nvratov hodnota vkonn funkce na stran serveru,
@li pznaky chyb.
Na klientsk stran ped odeslnm pkazu se nastavuje dle poteby pouze pole tae, ostatn sti jsou 
nevznamn. Po zpracovn pkazu na serveru a penosu zpt ke klientovi zstv obsah tae zachovn
(nap. pro kontrolu sprvnho provn odeslanhch a pijatch dat), dle vsledku operac jsou naplnny
dal sti.

Pro usnadnn manipulace s stmi stavovho slova jsou urena @ref cmd_utils "makra".
*/
class CCommand : public CBlobEx<unsigned> {
public:
  //! konstruktor
  /*!
  Konstruktor bez inicializace dat (binrn data jsou przdn).
  */
  CCommand(void) : CBlobEx<unsigned>() {}
  //! konstruktor s inicializac
  /*!
  Konstruktor s inicializac dat
  param[in] uiStatus hodnota stavovho slova
  param[in] pData ukazatel na pole binrnch dat
  param[in] uiLength velikost bimrnch dat
  */
  CCommand(unsigned uiStatus, const void *pData, unsigned uiLength) : CBlobEx<unsigned>(uiStatus, pData, uiLength) {}
  //! vtov typ pro druh pkazu
  /*!
  Kad poloka vtovho typu odpovd metod rozhran @ref CQuery. Pi zpracovn na serverov stran
  se pak dle jej hodnoty vyvol pslun metoda databzovho rozhran.
  */
  enum EQueryCmd {CMDADDCITY, CMDADDCITYALIAS, CMDDELETECITY, CMDFINDCITY, CMDCITYINFO, CMDADDBUSPLAN,
    CMDDELETEBUSPLAN, CMDFINDBUSPLAN, CMDBUSPLANINFO, CMDADDBUS, CMDDELETEBUS, CMDFINDBUS, CMDBUSINFO,
    CMDENABLEBUS, CMDDISABLEBUS, CMDNEWCUSTOMER, CMDASKSEATS, CMDLOCKSEATS, CMDUNLOCKSEATS, CMDNEXTBUS,
    CMDNEXTCITY, CMDNEXTPLAN, CMDLISTCITY, CMDLISTALIAS};
  //! hodnota stavovho slova
  /*! 
  Pro prci s jednotlivmi stmi stavovho slova je mono pout @ref cmd_utils "konstanty a inline funkce".
  @return aktuln hodnota stavovho slova
  */
  unsigned Status(void) const {return Get();}
};

//! bitov maska pro ta
/*! Slou pro oddlen (pomoc bitovho AND) hodnoty tae ze stavovho slova. */
const unsigned CMD_COUNTER_MASK = 0x0000ffff;
//! pidn nvratov hodnoty zpracovatelsk funkce
/*!
Pomoc inline funkce se uprav nvratov hodnota zpracovatelsk funkce na serveru ped pidnm 
(bitov OR) do vytvenho stavovho slova.
@param[in] bRetVal nvratov hodnota zpracovatelsk funkce
@return upraven hodnota pro vytvoen stavovho slova
*/
inline unsigned CMD_SETFUNCRESULT(BYTE bRetVal) {return bRetVal << 16;}
//! zskn nvratov hodnoty zpracovatelsk funkce
/*!
Pomoc inline funkce se ze stavovho slova zsk nvratov hodnota zpracovatelsk funkce na serveru.
@param[in] uiStatus stavov slovo
@return nvratov hodnota zpracovatelsk funkce
*/
inline BYTE CMD_GETFUNCRESULT(unsigned uiStatus) {return static_cast<BYTE>(uiStatus >> 16);}
//! pznak chyby voln zpracovatelsk funkce
/*!
Tato chyba nastane, pokud funkce pro zpracovn pkazu na stran serveru nen definovna.
*/
const unsigned CMD_FUNCFAIL_FLAG = 0x01000000;
//! pznak chyby komunikace
/*!
Pznak je nastaven, pokud se v prbhu komunikace klient-server detekuje poruen dat.
*/
const unsigned CMD_COMMFAIL_FLAG = 0x02000000;

#endif
