#define _CRT_SECURE_NO_DEPRECATE

#include <windows.h>
#include <process.h>
#include <string.h>
#include <list>

#pragma hdrstop

#include "MsgEndpoint.h"

static const char pszDefaultWndName[] = "MsgEndpointWnd";
static const char pszDefaultWndClass[] = "MsgEndpointWndClass";

CMsgEndpoint::CMsgEndpoint(const char *pszName, const char *pszClass)
{
  unsigned uiStrLen;

  m_hWnd = NULL;
  InitializeCriticalSection(&m_rLock);
  if(pszName == NULL || *pszName == '\0')
    pszName = pszDefaultWndName;
  uiStrLen = (unsigned)strlen(pszName);
  m_pszName = new char [uiStrLen + 1];
  strcpy(m_pszName, pszName);
  if(pszClass == NULL || *pszClass == '\0')
    pszClass = pszDefaultWndClass;
  uiStrLen = (unsigned)strlen(pszClass);
  m_pszClass = new char [uiStrLen + 1];
  strcpy(m_pszClass, pszClass);
  m_hReceivedEvent = CreateEvent(NULL, FALSE, FALSE, NULL); // nesignalizovan auto-reset udlost
  m_hEnabledEvent = CreateEvent(NULL, TRUE, TRUE, NULL); // signalizovan manual-reset udlost
  // vytvoen pracovnho vlkna a ekn na jeho skuten sputn a vytvoen okna
  m_hReadyEvent = CreateEvent(NULL, TRUE, FALSE, NULL); // nesignalizovan manual-reset udlost
  m_hThread = (HANDLE)_beginthreadex(NULL, 0, WorkThread, this, 0, NULL);
  WaitForSingleObject(m_hReadyEvent, INFINITE);
}

CMsgEndpoint::~CMsgEndpoint()
{
  // zruen okna, ekn na ukonen pracovnho vlkna
  DestroyWindow(m_hWnd);
  PostMessage(m_hWnd, WM_QUIT, 0, 0);
  WaitForSingleObject(m_hThread, INFINITE);
  // destrukce alokovanch zdroj
  DeleteCriticalSection(&m_rLock);
  delete [] m_pszName;
  delete [] m_pszClass;
  CloseHandle(m_hThread);
  CloseHandle(m_hReadyEvent);
  CloseHandle(m_hReceivedEvent);
  CloseHandle(m_hEnabledEvent);
}

bool CMsgEndpoint::Send(CMsgData *cMsg)
{
  COPYDATASTRUCT rCopyData;
  HWND hToWnd;

  hToWnd = cMsg->Get().m_hWnd;
  if(IsWindow(hToWnd)) {
    rCopyData.dwData = cMsg->Get().m_dwId;
    rCopyData.cbData = cMsg->DataLen();
    rCopyData.lpData = (LPVOID)cMsg->DataPointer();
    return SendMessage(hToWnd, WM_COPYDATA, (WPARAM)m_hWnd, (LPARAM)&rCopyData) != FALSE;
  }
  else
    return false;
}

HANDLE CMsgEndpoint::ReceivedEvent(void) const
{
  return m_hReceivedEvent;
}

unsigned CMsgEndpoint::Received(void)
{
  unsigned uiReceived;

  EnterCriticalSection(&m_rLock);
  uiReceived = (unsigned)size();
  LeaveCriticalSection(&m_rLock);
  return uiReceived;
}

bool CMsgEndpoint::Get(CMsgData *cMsg)
{
  bool fResult;

  EnterCriticalSection(&m_rLock);
  if(!empty()) {
    *cMsg = front();
    pop_front(); // zruen peten poloky z kontajneru
    if(!empty())
      SetEvent(m_hReceivedEvent); // v kontajneru dal platn data
    fResult = true;
  }
  else
    fResult = false;
  LeaveCriticalSection(&m_rLock);
  return fResult;
}

void CMsgEndpoint::Enable(bool fYesNo)
{
  if(fYesNo)
    SetEvent(m_hEnabledEvent);
  else
    ResetEvent(m_hEnabledEvent);
}

void CMsgEndpoint::ClearReceived(void)
{
  EnterCriticalSection(&m_rLock);
  clear();
  LeaveCriticalSection(&m_rLock);
}

LRESULT CALLBACK CMsgEndpoint::WindowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  CMsgEndpoint *cThis;
  COPYDATASTRUCT *prData;
  CMsgData *cData;
  RMsgData rMsgData;

  cThis = (CMsgEndpoint *)GetWindowLongPtr(hWnd, GWLP_USERDATA); // ukazatel this uchovn v uivatelskch datech okna
  switch(uMsg) {
    case WM_COPYDATA: // jedin zprva se zvltnm zpracovnm
      if(WaitForSingleObject(cThis->m_hEnabledEvent, 0) == WAIT_OBJECT_0) {
        prData = (COPYDATASTRUCT *)lParam;
        rMsgData.m_hWnd = (HWND)wParam;
        rMsgData.m_dwId = (DWORD)prData->dwData;
        cData = new CMsgData(rMsgData, prData->lpData, prData->cbData);
        EnterCriticalSection(&cThis->m_rLock);
        cThis->push_back(*cData);
        SetEvent(cThis->m_hReceivedEvent);
        LeaveCriticalSection(&cThis->m_rLock);
        delete cData;
        return TRUE;
      }
      else
        return FALSE;
    default:
      return DefWindowProc(hWnd, uMsg, wParam, lParam);
  }
}

unsigned int WINAPI CMsgEndpoint::WorkThread(LPVOID pThreadParams)
{
  CMsgEndpoint *cThis;
  WNDCLASSEX rClassInfo;
  MSG rMsg;
  HINSTANCE hInstance;

  cThis = (CMsgEndpoint *)pThreadParams;
  hInstance = (HINSTANCE)GetModuleHandle(NULL);
  // zaregistrovn tdy okna
  if(!GetClassInfoEx(hInstance, cThis->m_pszClass, &rClassInfo)) {
    rClassInfo.cbSize = sizeof(WNDCLASSEX);
    rClassInfo.style = 0;
    rClassInfo.cbClsExtra = 0;
    rClassInfo.cbWndExtra = 0;
    rClassInfo.hInstance = hInstance;
    rClassInfo.hIcon = NULL;
    rClassInfo.hIconSm = NULL;
    rClassInfo.hCursor = NULL;
    rClassInfo.hbrBackground = NULL;
    rClassInfo.lpszMenuName = NULL;
    rClassInfo.lpfnWndProc = (WNDPROC)cThis->WindowProc;
    rClassInfo.lpszClassName = cThis->m_pszClass;
    RegisterClassEx(&rClassInfo);
  }
  // vytvoen message-only okna
  cThis->m_hWnd = CreateWindowEx(0, cThis->m_pszClass, cThis->m_pszName, 0, 0, 0, 0, 0, HWND_MESSAGE, (HMENU)NULL, hInstance, NULL);
  SetWindowLongPtr(cThis->m_hWnd, GWLP_USERDATA, (LONG_PTR)cThis); // uloen this do uivatelskch dat okna
  SetEvent(cThis->m_hReadyEvent);
  // smyka zprv pouze pro toto vlkno a okno
  while(GetMessage(&rMsg, cThis->m_hWnd, 0, 0)) {
    TranslateMessage(&rMsg);
    DispatchMessage(&rMsg);
  }
  return 0; // pijato WM_QUIT v dsledku voln destruktoru
}
