#ifndef _MSGENDPOINT_H
#define _MSGENDPOINT_H

#include <windows.h>
#include <list>
#include "UtilClass.h"

/*!
@file MsgEndpoint.h
Deklarace tdy @ref CMsgEndpoint.
*/

//! struktura pro definici dodatench parametr
/*!
Tda @ref CMsgEndpoint obsahuje frontu pijatch zprv. Tato struktura slou jako parametr
ablony tdy, kter reprezentuj jednotliv zprvy (@ref CMsgData).
*/
struct RMsgData {
  //! handle okna pro odpov
  HWND m_hWnd;
  //! stavov slovo
  DWORD m_dwId;
};

//! definice typu pro uloen pijatch zprv
typedef CBlobEx<RMsgData> CMsgData;

//! tda pro obsluhu penosu zprv mezi procesy
/*!
Zprvy mezi procesy jsou peneny pomoc zprvy @b WM_COPYDATA operanho systmu obma smry.
K tomu mus na pijmac stran existovat okno, kter zprvu pijme a zpracuje, a vlkno, kter
zaruuje obsluhu ostatnch zprv zaslanch pomoc @c PostMessage. Odesln zprv probh
v blokujcm reimu (pomoc @c SendMessage). Tda samotn neprovd dnou dal manipulaci 
s penenmi informacemi, to je lohou vych vrstev.

Pijat zprvy jsou uchovvny v internm kontajneru, do kterho se sekvenn pidvaj.
Konzistence dat v kantajneru pi soubnm pstupu z vce vlken je zajitna kritickou sekc.
Dostupnost dat je signalizovna pomoc objektu jdra - udlosti.
*/
class CMsgEndpoint : private std::list<CMsgData> {
public:
  //! konstruktor
  /*!
  Konstruktor provd vytvoen a sputn pracovnho vlkna, kter zajiuje:
  @li registraci tdy okna, pokud je teba
  @li vytvoen nevizulnho okna urenho pouze pro pjem a zpracovn @c WM_COPYDATA
  (ostatn zprvy jsou pedny standardn obsluze)
  @li provdn smyky zprv pro dnou obsluhu @e post zprv

  @param[in] pszName jmno okna pro pjem zprv 
  @n Dleit na stran serveru pro nalezen pjemce zprv od klienta. Pokud je @e NULL,
  pouije se defaultn jmno.
  @param[in] pszClass jmno tdy okna, kter je vytvoeno pro pjem zprv
  @n Pro hodnotu @e NULL se pouije defaultn tda.
  */
  CMsgEndpoint(const char *pszName, const char *pszClass);
  //! destruktor
  /*!
  Pi destrukci objektu se provede zruen okna, ukonen smyky zprv v pracovnm vlkn 
  (pomoc zprvy @c WM_QUIT) a pok se na ukonen pracovnho vlkna. A pot nsleduje 
  dealokace vech zdroj.
  */
  ~CMsgEndpoint();
  //! odesln dat
  /*!
  Provede uloen pedanch dat do formtu zprvy @c WM_COPYDATA a jejich odesln pomoc
  @c SendMessage do clovho okna.
  @param[in] cMsg data zprvy k odesln (krom pole binrnch dat obsahuj i stavov slovo
  a handle pijmacho okna)
  @return spch operace (@e true) i selhn (@e false) nap. v dsledku neexistence
  pijmacho okna 
  */
  bool Send(CMsgData *cMsg);
  //! udlost o dostupnosti dat
  /*! @return handle udlosti, jej signalizovan stav zna ptomnost dat
  ve vnitnm kontajneru */
  HANDLE ReceivedEvent(void) const;
  //! poet pijatch zprv
  /*! @return poet zprv ve vnitnm kontajneru */
  unsigned Received(void);
  //! vyzvednut dat
  /*!
  Vyzvednut prvn zprvy z datovho kontajneru a pedn do pslun tdy.
  @param[out] cMsg data pijat zpravy, kter se skldaj z handle okna pro odpov, stavovho slova a
  vlastnho pole binrnch dat
  @return proveden operace - @e false v ppad przdnho kontajneru
  */
  bool Get(CMsgData *cMsg);
  //! pozastaven/obnoven pjmu zprv
  /*!
  Po sputn pracovnho vlkna a vytvoen okna je toto pipraveno pijmat a zpracovvat zprvy
  @c WM_COPYDATA. Toto lze doasn pozastavit a pozdji opt obnovit. V ppad pozastavenho pjmu
  je nvratov hodnota metody @ref Send @e false. Jinch zprv se pozastaven netk, jsou zpracovvny
  normln.
  @param[in] fYesNo povolen (@e true) nebo pozastaven (ve false) pjmu zprv
  */
  void Enable(bool fYesNo);
  //! vyprzdnn kontajneru pijatch zprv
  void ClearReceived(void);
private:
  char *m_pszName;
  char *m_pszClass;
  HWND m_hWnd;
  static LRESULT CALLBACK WindowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
  HANDLE m_hThread;
  HANDLE m_hReadyEvent;
  HANDLE m_hReceivedEvent;
  HANDLE m_hEnabledEvent;
  static unsigned int WINAPI WorkThread(LPVOID pThreadParams);
  // zmek pro ochranu dat v kontajneru
  CRITICAL_SECTION m_rLock;
};

#endif
