#define _CRT_SECURE_NO_DEPRECATE

#include <winsock2.h> // mus bt ped <windows.h>
#include <windows.h>
#include <string.h>
#include <stdio.h>
#include <ws2tcpip.h>

#pragma hdrstop

#include "SocketEndpoint.h"
#include "Const.h"

#pragma comment(lib, "ws2_32.lib") // knihovna s tmto jmnem existuje v 32-bit i 64-bit verzi

CSocketEndpoint::CSocketEndpoint(const char *pszLocation)
{
  WSADATA rData;
  char *pszAddress;
  char *pszPort;
  addrinfo rHints;
  addrinfo *prHosts;
  unsigned uiStrLen;

  m_hSocket = INVALID_SOCKET;
  m_fInitialized = WSAStartup(MAKEWORD(2, 2), &rData) == 0;
  // zpracovn adresy ("localhost", "*:5698", "comp1.company.tld:32690", "127.0.0.1:25100", apod.)
  m_ulAddress = INADDR_ANY;
  m_usPort = htons(atoi(TCP_DEFAULT_PORT));
  uiStrLen = pszLocation == NULL ? 0 : (unsigned)strlen(pszLocation);
  if(uiStrLen != 0) {
    pszAddress = new char [uiStrLen + 1];
    strcpy(pszAddress, pszLocation);
    // rozdlen etzce na adresu a port
    pszPort = strchr(pszAddress, ':');
    if(pszPort != NULL) {
      *pszPort = '\0';
      pszPort++;
      if(*pszPort == '\0')
        pszPort = NULL;
    }
    // pevod jmna na IP adresu a port
    memset(&rHints, 0, sizeof(addrinfo));
    rHints.ai_family = AF_INET;
    rHints.ai_socktype = SOCK_STREAM;
    rHints.ai_protocol = IPPROTO_TCP;
    prHosts = NULL;
    if(*pszAddress == '\0' || *pszAddress == '*') {
      // libovoln IP adresa
      if(pszPort != NULL)
        m_usPort = htons(atoi(pszPort));
    }
    else if(getaddrinfo(pszAddress, (pszPort == NULL) ? TCP_DEFAULT_PORT : pszPort, &rHints, &prHosts) == 0) {
      m_ulAddress = ((sockaddr_in *)prHosts->ai_addr)->sin_addr.s_addr;
      m_usPort = ((sockaddr_in *)prHosts->ai_addr)->sin_port;
      freeaddrinfo(prHosts);
    }
    delete [] pszAddress;
  }
}

CSocketEndpoint::~CSocketEndpoint()
{
  CloseSocket();
  if(m_fInitialized)
    WSACleanup();
}

SOCKET CSocketEndpoint::Socket(void) const
{
  return m_hSocket;
}

bool CSocketEndpoint::Valid(void) const
{
  return m_hSocket != INVALID_SOCKET;
}

void CSocketEndpoint::Address(sockaddr_in *prAddress)
{
  memset(prAddress, 0, sizeof(sockaddr_in));
  prAddress->sin_family = AF_INET;
  prAddress->sin_addr.s_addr = m_ulAddress;
  prAddress->sin_port = m_usPort;
}

void CSocketEndpoint::OpenSocket(void)
{
  BOOL fOption;

  if(m_fInitialized) {
    m_hSocket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
    if(m_hSocket != INVALID_SOCKET) {
      fOption = TRUE;
      setsockopt(m_hSocket, SOL_SOCKET, SO_REUSEADDR, (char *)&fOption, sizeof(BOOL));
      setsockopt(m_hSocket, SOL_SOCKET, SO_KEEPALIVE, (char *)&fOption, sizeof(BOOL));
    }
  }
}

void CSocketEndpoint::CloseSocket(void)
{
  if(m_hSocket != INVALID_SOCKET) {
    closesocket(m_hSocket);
    m_hSocket = INVALID_SOCKET;
  }
}

int CSocketEndpoint::Send(const char *pcData, unsigned uiLength)
{
  return ::Send(Socket(), pcData, uiLength);
}

int CSocketEndpoint::Receive(char *pcData, unsigned uiLength)
{
  return ::Receive(Socket(), pcData, uiLength);
}

// --- exportovan funkce pro odesln a pjem dlouhch blok dat (dje se po blocch)
const unsigned BLOCK_LENGTH = 512;

int Send(SOCKET hSocket, const char *pcData, unsigned uiLength)
{
  int iResult;
  int iLength;

  iLength = (int)uiLength;
  while(uiLength >= BLOCK_LENGTH) {
    iResult = send(hSocket, pcData, BLOCK_LENGTH, 0);
    if(iResult != (int)BLOCK_LENGTH)
      return iResult;
    pcData += BLOCK_LENGTH;
    uiLength -= BLOCK_LENGTH;
  }
  if(uiLength != 0) {
    iResult = send(hSocket, pcData, uiLength, 0);
    if(iResult != (int)uiLength)
      return iResult;
  }
  return iLength;
}

int Receive(SOCKET hSocket, char *pcData, unsigned uiLength)
{
  int iResult;
  int iLength;

  iLength = (int)uiLength;
  while(uiLength >= BLOCK_LENGTH) {
    iResult = recv(hSocket, pcData, BLOCK_LENGTH, 0);
    if(iResult != (int)BLOCK_LENGTH)
      return iResult;
    pcData += BLOCK_LENGTH;
    uiLength -= BLOCK_LENGTH;
  }
  if(uiLength != 0) {
    iResult = recv(hSocket, pcData, uiLength, 0);
    if(iResult != (int)uiLength)
      return iResult;
  }
  return iLength;
}
