#ifndef _SOCKETENDPOINT_H
#define _SOCKETENDPOINT_H

#include <winsock2.h>

/*!
@file SocketEndpoint.h
Deklarace tdy @ref CSocketEndpoint s podporou zkladnch vlastnost pi prci se sokety.
*/

//! obsluha TCP zakonen
/*!
Tda zapozduje spolen vlastnosti pijmac a vyslac sti soketu, pomoc kterch probh
vzjemn komunikace.
*/
class CSocketEndpoint {
public:
  //! konstruktor
  /*!
  Provd inicializaci knihovny pro podporu soket ve Windows a pevd poadovan URL
  na IP adresu a port pomoc DNS.
  @param[in] pszLocation definice msta svzanho s vnitnm soketem tdy
  @n Definice msta se skld z adresy a volielnho portu oddlenho dvojtekou. Defaultn port je 
  uren konstantou @ref TCP_DEFAULT_PORT. Adresa me bt v selnm formtu nebo je urena pomoc 
  domnovho jmna. Pklady rozeznvanch formt:
  @li <tt>"localhost"</tt> - spojen na mstn smyce a defaultnm portu,
  @li <tt>"127.0.0.1:25100"</tt> - spojen na mstn smyce a portu 25100,
  @li <tt>":25100" "*:25100"</tt> - spojen na libovoln adrese potae a portu 25100, co m smysl 
  pro naslouchac soket na stran serveru,
  @li <tt>"comp32.company.tld"</tt> - msto uren domnovm jmnem, port defaultn.
  */
  CSocketEndpoint(const char *pszLocation);
  //! destruktor
  /*! Uzavr ppadn oteven soket a ukonuje prci se soketovou knihovnou. */
  ~CSocketEndpoint();
  //! popisova soketu
  /*! @return popisova platnho soketu nebo @c INVALID_SOCKET */
  SOCKET Socket(void) const;
  //! platnost soketu
  /*! @return @e true, pokud je vnitn soket oteven a platn */
  bool Valid(void) const;
  //! naplnn struktury IP adresou a portem
  /*! @param[out] prAddress adresa struktury pro naplnn daji zskanmi pi voln konstruktoru */
  void Address(sockaddr_in *prAddress);
  //! oteven soketu
  /*! 
  IP adresa a port jsou definovny pi voln konstruktoru, na jejich zklad je oteven soket
  pro komunikaci pomoc TCP a jsou nastaveny jeho zkladn vlastnosti.
  */
  void OpenSocket(void);
  //! uzaven soketu
  void CloseSocket(void);
  //! odesln dat
  /*!
  Data jsou odeslna pomoc vnitnho soketu, nen provdna dn jin manipulace se soketem.
  Vt rozsah dat se odesl po blocch.
  @param[in] pcData adresa bloku dat k odesln
  @param[in] uiLength dlka dat
  @return velikost skuten odeslanch dat (0 v ppad chyby, nap. neoteven soketu apod.)
  */
  int Send(const char *pcData, unsigned uiLength);
  //! pjem dat
  /*!
  Pomoc vnitnho soketu jsou pijmna data (vt rozsah po blocch) a ukldna do uren pamti.
  @param[out] pcData adresa bloku pamti k uloen pijatch dat
  @param[in] uiLength dlka bloku pamti (tj. maximln mon poet byt k uloen)
  @return velikost skuten pijatch dat (0 v ppad chyby, nap. neoteven soketu apod.)
  */
  int Receive(char *pcData, unsigned uiLength);
private:
  bool m_fInitialized;
  SOCKET m_hSocket;
  unsigned long m_ulAddress;
  unsigned short m_usPort;
};

//! odesln dat
/*!
Data jsou odeslna pomoc zadanho soketu, nen provdna dn jin manipulace se soketem, 
tj. soket mus bt dn oteven a pipojen. Vt rozsah dat se odesl po blocch.
@param[in] hSocket popisova soketu pro komunikaci
@param[in] pcData adresa bloku dat k odesln
@param[in] uiLength dlka dat
@return velikost skuten odeslanch dat (0 v ppad chyby, nap. neoteven soketu apod.)
*/
extern int Send(SOCKET hSocket, const char *pcData, unsigned uiLength);
//! pjem dat
/*!
Pjem dat probh pomoc urenho soketu (vt rozsah po blocch). Soket mus bt dn
oteven a pipojen.
@param[in] hSocket popisova soketu pro komunikaci
@param[out] pcData adresa bloku pamti k uloen pijatch dat
@param[in] uiLength dlka bloku pamti (tj. maximln mon poet byt k uloen)
@return velikost skuten pijatch dat (0 v ppad chyby, nap. neoteven soketu apod.)
*/
extern int Receive(SOCKET hSocket, char *pcData, unsigned uiLength);

#endif
