#include <windows.h>
#include <string.h>

#pragma hdrstop

#include "UtilClass.h"

// === CBlob ===============================================================
CBlob::CBlob(void) : m_uiLength(0), m_pbData(NULL)
{
}

CBlob::CBlob(const void *pData, unsigned uiLength) : m_uiLength(uiLength)
{
  m_pbData = NULL;
  Set(pData, uiLength);
}

CBlob::CBlob(const CBlob *cSrc) : m_uiLength(0), m_pbData(NULL)
{
  if(cSrc != NULL)
    *this = *cSrc;
}

CBlob::CBlob(const CBlob &cSrc) : m_uiLength(0), m_pbData(NULL)
{
  *this = cSrc;
}

CBlob::~CBlob()
{
  delete [] m_pbData;
}

void CBlob::Set(const void *pData, unsigned uiLength)
{
  delete [] m_pbData;
  if(pData == NULL || uiLength == 0) {
    m_pbData = NULL;
    m_uiLength = 0;
  }
  else {
    m_pbData = new unsigned char [uiLength];
    memcpy(m_pbData, pData, uiLength);
    m_uiLength = uiLength;
  }
}

void CBlob::Clear(void)
{
  delete [] m_pbData;
  m_pbData = NULL;
  m_uiLength = 0;
}

unsigned CBlob::DataLen(void) const
{
  return m_uiLength;
}

unsigned CBlob::CopyData(void *pBuffer, unsigned uiLength, unsigned uiOffset) const
{
  if(uiOffset + uiLength > m_uiLength)
    uiLength = m_uiLength - uiOffset;
  memcpy(pBuffer, m_pbData + uiOffset, uiLength);
  return uiLength;
}

const void *CBlob::DataPointer(void) const
{
  return m_pbData;
}

CBlob &CBlob::operator=(const CBlob &cRight)
{
  if(this != &cRight) // pi piazen sob sammu nedlej nic
    Set(cRight.m_pbData, cRight.m_uiLength);
  return *this;
}

// === CReadWriteLock ======================================================
CReadWriteLock::CReadWriteLock(unsigned uiMaxReaders)
{
  m_uiMaxReaders = uiMaxReaders;
  m_uiReaders = 0;
  m_uiWriters = 0;
  m_fWriter = false;
  InitializeCriticalSection(&m_rLock);
}

CReadWriteLock::CReadWriteLock(const CReadWriteLock &cSource)
{
  m_uiMaxReaders = cSource.m_uiMaxReaders;
  // inicializace - dn ten, dn zpis
  m_uiReaders = 0;
  m_uiWriters = 0;
  m_fWriter = false;
  InitializeCriticalSection(&m_rLock); // vlastn kritick sekce
}

CReadWriteLock::~CReadWriteLock()
{
  DeleteCriticalSection(&m_rLock);
}

void CReadWriteLock::EnterRead(void)
{
  bool fCan;

  while(true) {
    EnterCriticalSection(&m_rLock);
    // pstup ke ten povolen, pokud neprobh i nen poadovn zpis a poet ten nen vyerpn
    if(m_uiWriters != 0 || m_uiReaders == m_uiMaxReaders)
      fCan = false;
    else {
      m_uiReaders++; // zapoten nov povolenho tene
      fCan = true;
    }
    LeaveCriticalSection(&m_rLock);
    if(fCan)
      return;
    Sleep(0); // pedej zbytek asu ostatnm vlknm
  }
}

void CReadWriteLock::LeaveRead(void)
{
  EnterCriticalSection(&m_rLock);
  m_uiReaders--; // uvolnn pozice tene
  LeaveCriticalSection(&m_rLock);
}

void CReadWriteLock::EnterWrite(void)
{
  bool fCan;

  EnterCriticalSection(&m_rLock);
  m_uiWriters++; // zaregistruj poadavek na zpis
  LeaveCriticalSection(&m_rLock);
  while(true) {
    EnterCriticalSection(&m_rLock);
    // pstup pro zpis je povolen, pokud neprobh jin zpis ani ten
    if(m_fWriter || m_uiReaders != 0)
      fCan = false;
    else {
      m_fWriter = true; // nastaven pznaku aktivnho zpisu
      fCan = true;
    }
    LeaveCriticalSection(&m_rLock);
    if(fCan)
      return;
    Sleep(0); // pedej zbytek asu ostatnm vlknm
  }
}

void CReadWriteLock::LeaveWrite(void)
{
  EnterCriticalSection(&m_rLock);
  m_uiWriters--; // uvolnn poadavku na zpis
  m_fWriter = false; // zruen pznaku aktivnho pstupu pro zpis
  LeaveCriticalSection(&m_rLock);
}

CReadWriteLock &CReadWriteLock::operator=(const CReadWriteLock &cRight)
{
  // dn innost
  return *this;
}

// === CAutoIncrement ======================================================
CAutoIncrement::CAutoIncrement(unsigned uiStart)
{
  m_uiCurrent = uiStart;
  InitializeCriticalSection(&m_rLock);
}

CAutoIncrement::~CAutoIncrement()
{
  DeleteCriticalSection(&m_rLock);
}

unsigned CAutoIncrement::Next(void)
{
  unsigned uiResult;

  EnterCriticalSection(&m_rLock);
  uiResult = ++m_uiCurrent;
  LeaveCriticalSection(&m_rLock);
  return uiResult;
}

void CAutoIncrement::Set(unsigned uiValue)
{
  EnterCriticalSection(&m_rLock);
  if(uiValue > m_uiCurrent)
    m_uiCurrent = uiValue;
  LeaveCriticalSection(&m_rLock);
}
