#ifndef _ADMIN_H
#define _ADMIN_H

#include <iosfwd>

/*!
@file Admin.h
Zde je deklarovna tda @ref CAdmin, kter tvo sprvcovsk rozhran databze.
*/

//! informace o databzi
struct RDbInfo {
  //! poet mst
  unsigned uiCities;
  //! poet alias mst
  unsigned uiCityAliases;
  //! poet pln autobus
  unsigned uiPlans;
  //! poet autobus
  unsigned uiBuses;
};

//! sprvcovsk rozhran databze
/*!
Tda poskytuje deklaraci (ist virtulnch) metod urench pro sprvu databze. Kad implementace
databze mus krom krom klientskho rozhran @ref CQuery definovat i metody pro jej sprvu. Na
rozdl od zmnnho klientskho rozhran se nepedpokld vzdlen (meziprocesov) voln metod
rozhran pro sprvu.
*/
class CAdmin {
public:
  //! destruktor
  /*!
  Destruktor je deklarovn jako virtuln, sm neprovd dnou innost.
  */
  virtual ~CAdmin() = 0;
  //! zloha databze
  /*!
  Proveden zlohy databze do vstupnho proudu je konzistentnm zachycenm jejho obrazu v njakm
  okamiku. K tomu zsk vhradn pstup k databzi (pro @ref CMemoryDb viz @ref db_lock).
  @param[in] cStream vstupn proud pro proveden zlohy. Pedpokld se, e proud je ji oteven
  do reimu umoujc binrn zpis dat. Pitom je lhostejno, zda je spojen se souborem, TCP soketem
  i jinm zazenm. Po dokonen operace nen proud uzaven (je pouze synchronizovn), to mus 
  uinit volajc.
  @return vsledek operace
  @note Problematika zlohovn databze, obnovy dat, jejich formtu atd. je podrobnji popsna 
  na @ref backup "zvltn strnce".
  */
  virtual bool Backup(std::ostream *cStream) = 0;
  //! informace o databzi
  /*!
  Zskn potu objekt jednotlivch typ v databzi.
  @param[in] prInfo ukazatel na struktutu, kter bude naplnna poadovanmi daji.
  @return vsledek operace
  */
  virtual bool Info(RDbInfo *prInfo) = 0;
  //! akce pi sputn serveru
  /*!
  Uivatelsk akce (specifick dle druhu pouit databze), kter se provd tsn ped sputnm
  komunikanch server - pkaz @b START.
  */
  virtual void OnStart(void) = 0;
  //! akce pi zastaven serveru
  /*!
  Uivatelsk akce (specifick dle druhu pouit databze), kter se provd tsn po zastaven
  komunikanch server - pkaz @b STOP.
  */
  virtual void OnStop(void) = 0;
};

inline CAdmin::~CAdmin() {}

#endif
