#include <istream>
#include <string>
#include <string.h>

#pragma hdrstop

#include "Backup.h"

static const unsigned c_uiHeader = 0x2b2b2b23;

// --- CBkHeader

static const unsigned auiHeaderValue[] = {
  0x20206244, // eDatabase
  0x735f6943, // eCities
  0x735f6c41, // eAliases
  0x735f6c50, // ePlans
  0x735f7542, // eBuses
  0x79746943, // eCity
  0x61696c41, // eAlias
  0x6e616c50, // ePlan
  0x20737542, // eBus
  0x74616553, // eSeat
  0x20646e45, // eEnd
};

unsigned CBkHeader::HeaderValue(EObject eObject)
{
  return auiHeaderValue[static_cast<unsigned>(eObject)];
}

// --- CBackup

CBackup::CBackup(std::ostream *cStream) : m_cStream(cStream)
{
  CheckStream();
}

void CBackup::Header(EObject eObject)
{
  Unsigned(c_uiHeader);
  Unsigned(HeaderValue(eObject));
}

void CBackup::Bool(bool fValue)
{
  unsigned uiValue = fValue ? 1 : 0;
  m_cStream->write((const char *)&uiValue, sizeof(unsigned));
  CheckStream();
}

void CBackup::Unsigned(unsigned uiValue)
{
  m_cStream->write((const char *)&uiValue, sizeof(unsigned));
  CheckStream();
}

void CBackup::Block(const void *pAddress, unsigned uiSize)
{
  m_cStream->write((const char *)pAddress, uiSize);
  CheckStream();
}

void CBackup::String(const char *pszValue)
{
  m_cStream->write(pszValue, strlen(pszValue) + 1);
  CheckStream();
}

void CBackup::String(const std::string &strValue)
{
  m_cStream->write(strValue.c_str(), strValue.length() + 1);
  CheckStream();
}

void CBackup::CheckStream()
{
  if(!m_cStream)
    throw std::ios::failure("backup");
}

// --- CRestore

CRestore::CRestore(std::istream *cStream) : m_cStream(cStream)
{
  CheckStream();
}

bool CRestore::Bool(void)
{
  unsigned uiValue;
  m_cStream->read((char *)&uiValue, sizeof(unsigned));
  CheckStream();
  return uiValue != 0;
}

unsigned CRestore::Unsigned(void)
{
  unsigned uiValue;
  m_cStream->read((char *)&uiValue, sizeof(unsigned));
  CheckStream();
  return uiValue;
}

void CRestore::Block(void *pAddress, unsigned uiSize)
{
  m_cStream->read((char *)pAddress, uiSize);
  CheckStream();
}

void CRestore::String(std::string &strValue)
{
  char cChar;
  strValue.clear();
  while(m_cStream->get(cChar) && cChar != '\0')
    strValue.push_back(cChar);
  CheckStream();
}

void CRestore::Header(EObject eObject)
{
  if(Unsigned() != c_uiHeader || Unsigned() != HeaderValue(eObject))
    throw std::ios::failure("restore");
}

void CRestore::CheckStream()
{
  if(m_cStream->fail())
    throw std::ios::failure("restore");
}
