#include <map>
#include <string>
#include <istream>
#include <algorithm>

#pragma hdrstop

#include "City.h"
#include "Backup.h"
#include "../../Common/Query.h"
#include "../../Common/UtilClass.h"

static const unsigned MAX_READERS = 100;

// ------------- CCity ------------------------------------------------

CCity::CCity(const RCityInfo *prInfo) : RCityInfo(*prInfo), CReadWriteLock(MAX_READERS)
{
  m_uiLinesOut = 0;
  m_uiLinesIn = 0;
}

CCity::CCity(std::istream *cStream) : CReadWriteLock(MAX_READERS)
{
  CRestore cRestore(cStream);
  cRestore.Header(CBkHeader::eCity);
  uiId = cRestore.Unsigned();
  cRestore.String(strName);
  m_uiLinesOut = cRestore.Unsigned();
  m_uiLinesIn = cRestore.Unsigned();
}

CCity::~CCity()
{
}

void CCity::GetInfo(RCityInfo *prInfo)
{
  *prInfo = *static_cast<RCityInfo *>(this);
}

unsigned CCity::LinesOut(void)
{
  unsigned uiResult;

  EnterRead();
  uiResult = m_uiLinesOut;
  LeaveRead();
  return uiResult;
}

void CCity::LinesOut(int uiChange)
{
  EnterWrite();
  m_uiLinesOut += uiChange;
  LeaveWrite();
}

unsigned CCity::LinesIn(void)
{
  unsigned uiResult;

  EnterRead();
  uiResult = m_uiLinesIn;
  LeaveRead();
  return uiResult;
}

void CCity::LinesIn(int uiChange)
{
  EnterWrite();
  m_uiLinesIn += uiChange;
  LeaveWrite();
}

void CCity::Backup(std::ostream *cStream) const
{
  CBackup cBackup(cStream);
  cBackup.Header(CBkHeader::eCity);
  cBackup.Unsigned(uiId);
  cBackup.String(strName);
  cBackup.Unsigned(m_uiLinesOut);
  cBackup.Unsigned(m_uiLinesIn);
}

// ------------- CCities ----------------------------------------------

bool CityIdLess(const CCity &cCity1, const CCity &cCity2)
{
  return cCity1.Id() < cCity2.Id();
}

CCities::CCities(void) : std::set<CCity, FnCityIdLess>(CityIdLess)
{
}

CCities::CCities(std::istream *cStream) : std::set<CCity, FnCityIdLess>(CityIdLess)
{
  CRestore cRestore(cStream);
  cRestore.Header(CBkHeader::eCities);
  Set(cRestore.Unsigned()); // potadlo
  unsigned uiItems = cRestore.Unsigned(); // poet poloek
  for(unsigned i = 0; i < uiItems; ++i)
    insert(CCity(cStream));
}

CCities::~CCities()
{
}

const CCity *CCities::Add(const char *pszName)
{
  RCityInfo rInfo;

  rInfo.uiId = Next(); // nov msto s uniktnm slem
  rInfo.strName = pszName;
  return &(*insert(CCity(&rInfo)).first); // vloen do kontajneru
}

bool CCities::Delete(unsigned uiId)
{
  RCityInfo rInfo;

  rInfo.uiId = uiId;
  return erase(CCity(&rInfo)) != 0;
}

const CCity *CCities::Find(unsigned uiId)
{
  RCityInfo rInfo;
  rInfo.uiId = uiId;
  std::set<CCity, FnCityIdLess>::iterator i = find(CCity(&rInfo));
  return i == end() ? NULL : &(*i); // vrac ukazatel na nalezenou tdu
}


unsigned CCities::GetNextId(unsigned uiId)
{
  RCityInfo rInfo;
  rInfo.uiId = uiId;
  std::set<CCity, FnCityIdLess>::iterator i = upper_bound(CCity(&rInfo));
  return i == end() ? 0 : i->Id();
}

unsigned CCities::Count(void) const
{
  return static_cast<unsigned>(size());
}

void CCities::Clear(void)
{
  clear();
}

void CCities::Backup(std::ostream *cStream)
{
  CBackup cBackup(cStream);
  cBackup.Header(CBkHeader::eCities);
  cBackup.Unsigned(Next()); // potadlo
  cBackup.Unsigned(static_cast<unsigned>(size())); // poet poloek
  for_each(begin(), end(), [=](const CCity &cCity){cCity.Backup(cStream);});
}
