#include <map>
#include <string>
#include <istream>
#include <algorithm>

#pragma hdrstop

#include "Plan.h"
#include "Backup.h"
#include "../../Common/Query.h"
#include "../../Common/UtilClass.h"

static const unsigned MAX_READERS = 100;

// ------------- CPlan ------------------------------------------------

CPlan::CPlan(const RPlanInfo *prInfo, const CBlobEx<std::string> *cBlob) : RPlanInfo(*prInfo), CBlobEx<std::string>(cBlob), CReadWriteLock(MAX_READERS)
{
  m_uiBuses = 0;
}

CPlan::CPlan(std::istream *cStream) : CReadWriteLock(MAX_READERS)
{
  CRestore cRestore(cStream);
  cRestore.Header(CBkHeader::ePlan);
  uiId = cRestore.Unsigned();
  cRestore.String(strName);
  uiSeats = cRestore.Unsigned();
  m_uiBuses = cRestore.Unsigned();
  std::string strPlanTxt;
  cRestore.String(strPlanTxt); // textov pln
  unsigned uiDataLen = cRestore.Unsigned();
  char *pcData = new char [uiDataLen];
  try {
    cRestore.Block(pcData, uiDataLen); // binrn pln
  }
  catch(std::ios::failure &) {
    delete [] pcData;
    throw; // pam sprvn uvolnna, pedej vjimku dle
  }
  Set(strPlanTxt, pcData, uiDataLen);
  delete [] pcData;
}

CPlan::~CPlan()
{
}

void CPlan::GetInfo(RPlanInfo *prInfo)
{
  *prInfo = *static_cast<RPlanInfo *>(this);
}

unsigned CPlan::Buses(void)
{
  unsigned uiResult;

  EnterRead();
  uiResult = m_uiBuses;
  LeaveRead();
  return uiResult;
}

void CPlan::Buses(int uiChange)
{
  EnterWrite();
  m_uiBuses += uiChange;
  LeaveWrite();
}

void CPlan::Data(CBlobEx<std::string> *cData)
{
  if(cData != NULL)
    *cData = *static_cast<CBlobEx<std::string> *>(this);
}

void CPlan::Backup(std::ostream *cStream) const
{
  CBackup cBackup(cStream);
  cBackup.Header(CBkHeader::ePlan);
  cBackup.Unsigned(uiId);
  cBackup.String(strName);
  cBackup.Unsigned(uiSeats);
  cBackup.Unsigned(m_uiBuses);
  cBackup.String(Get()); // textov pln
  cBackup.Unsigned(DataLen());
  cBackup.Block(DataPointer(), DataLen()); // binrn pln
}

// ------------- CPlans -----------------------------------------------

bool PlanIdLess(const CPlan &cPlan1, const CPlan &cPlan2)
{
  return cPlan1.Id() < cPlan2.Id();
}

CPlans::CPlans(void) : std::set<CPlan, FnPlanIdLess>(PlanIdLess)
{
}

CPlans::CPlans(std::istream *cStream) : std::set<CPlan, FnPlanIdLess>(PlanIdLess)
{
  CRestore cRestore(cStream);
  cRestore.Header(CBkHeader::ePlans);
  Set(cRestore.Unsigned()); // potadlo
  unsigned uiItems = cRestore.Unsigned(); // poet poloek
  for(unsigned i = 0; i < uiItems; ++i)
    insert(CPlan(cStream));
}

CPlans::~CPlans()
{
}

const CPlan *CPlans::Add(const char *pszName, unsigned uiSeats, const CBlobEx<std::string> *cBlob)
{
  RPlanInfo rInfo;

  rInfo.uiId = Next(); // nov pln s uniktnm slem
  rInfo.strName = pszName;
  rInfo.uiSeats = uiSeats;
  return &(*insert(CPlan(&rInfo, cBlob)).first); // vloen do kontajneru
}

bool CPlans::Delete(unsigned uiId)
{
  RPlanInfo rInfo;

  rInfo.uiId = uiId;
  return erase(CPlan(&rInfo, NULL)) != 0;
}

const CPlan *CPlans::Find(unsigned uiId)
{
  RPlanInfo rInfo;
  rInfo.uiId = uiId;
  std::set<CPlan, FnPlanIdLess>::iterator i = find(CPlan(&rInfo, NULL));
  return i == end() ? NULL : &(*i); // vrac ukazatel na nalezenou tdu
}

unsigned CPlans::GetNextId(unsigned uiId)
{
  RPlanInfo rInfo;
  rInfo.uiId = uiId;
  std::set<CPlan, FnPlanIdLess>::iterator i = upper_bound(CPlan(&rInfo, NULL));
  return i == end() ? 0 : i->Id();
}

unsigned CPlans::Count(void) const
{
  return static_cast<unsigned>(size());
}

void CPlans::Clear(void)
{
  clear();
}

void CPlans::Backup(std::ostream *cStream)
{
  CBackup cBackup(cStream);
  cBackup.Header(CBkHeader::ePlans);
  cBackup.Unsigned(Next()); // potadlo
  cBackup.Unsigned(static_cast<unsigned>(size())); // poet poloek
  for_each(begin(), end(), [=](const CPlan &cPlan){cPlan.Backup(cStream);});
}
