#include <istream>
#include <time.h>

#pragma hdrstop

#include "Seat.h"
#include "Backup.h"

const unsigned FOREVER = 0;

CSeat::CSeat(void) : m_fReserved(false), m_tExpiry(FOREVER), m_uiCustomer(0)
{
}

CSeat::CSeat(std::istream *cStream)
{
  CRestore cRestore(cStream);
  cRestore.Header(CBkHeader::eSeat);
  m_fReserved = cRestore.Bool();
  cRestore.Block(&m_tExpiry, sizeof(time_t));
  m_uiCustomer = cRestore.Unsigned();
}

CSeat::~CSeat()
{
}

bool CSeat::Reserve(unsigned uiCustomer)
{
  if(!Reserved(time(NULL)) || (uiCustomer == m_uiCustomer && m_tExpiry != FOREVER)) {
    // nebylo rezervovno nebo bylo rezervovno doasn stvajcm zkaznkem
    m_fReserved = true;
    m_tExpiry = FOREVER;
    m_uiCustomer = uiCustomer;
    return true;
  }
  else
    return false;
}

bool CSeat::ReserveFor(unsigned uiCustomer, unsigned uiSeconds)
{
  time_t tNow;

  tNow = time(NULL);
  if(!Reserved(tNow)) {
    m_fReserved = true;
    m_tExpiry = tNow + uiSeconds;
    m_uiCustomer = uiCustomer;
    return true;
  }
  else
    return false;
}

bool CSeat::Free(unsigned uiCustomer)
{
  bool fReserved;

  fReserved = Reserved(time(NULL));
  if(!fReserved || uiCustomer != m_uiCustomer)
    return false;
  else {
    // zruen rezervace me provst pouze zkaznk, kter ji uinil
    m_fReserved = false;
    return true;
  }
}

bool CSeat::Reserved(void)
{
  return Reserved(time(NULL));
}

bool CSeat::Reserved(time_t tNow)
{
  // rezervovno trvale nebo as doasn rezervace ji vyprel
  return m_fReserved && (m_tExpiry == FOREVER || tNow <= m_tExpiry);
}

void CSeat::Backup(std::ostream *cStream) const
{
  CBackup cBackup(cStream);
  cBackup.Header(CBkHeader::eSeat);
  cBackup.Bool(m_fReserved);
  cBackup.Block(&m_tExpiry, sizeof(time_t));
  cBackup.Unsigned(m_uiCustomer);
}
