#ifndef _SEAT_H
#define _SEAT_H

/*!
@file Seat.h
Deklarace tdy jednoho sedadla v autobusu.
*/

#include <iosfwd>
#include <time.h>

//! sedadlo v autobusu
/*!
Tda slou jako rozhran jednotlivch sedadel. Zajiuje sprvn rezervovn a uvolovn sedadel. 
Rezervaci sedadla je mon provst pouze doasn (na omezen asov sek) co umon zkaznkovi 
potvrzen poadavku bez nebezpe, e poadovan sedadla mezitm rezervuje nkdo jin. Pot me 
uren zkaznk rezervaci potvrdit (tj. nastavit ji jako asov neomezenou) nebo odmtnout. Pokud 
se tak nestane bhem uren "ochrann" doby, je sedadlo opt pokldno za voln.
*/
class CSeat {
public:
  //! konstruktor
  /*! Sedadlo je vytvoeno jako nerezervovan. */
  CSeat(void);
  //! naten z proudu
  /*!
  Tda je vytvoena natenm vech poloek z proudu.
  @param[in] cStream vstupn proud s binrnmi daty
  */
  CSeat(std::istream *cStream);
  //! destruktor
  ~CSeat();
  //! konen rezervace
  /*!
  Zarezervovn sedadla napod, tj. do doby odjezdu autobusu.
  @param[in] uiCustomer jedinen ID zkaznka, nutn pro ppadnou deaktivaci rezervace
  @return @e true pi spchu
  */
  bool Reserve(unsigned uiCustomer);
  //! asov omezen rezervace
  /*!
  Rezervace sedadla, kter po uritm ase vypr.
  @param[in] uiCustomer jedinen ID zkaznka, nutn pro ppadnou deaktivaci rezervace
  @param[in] uiSeconds dlka trvn rezervace ve vteinch
  @return @e true pi spchu
  */
  bool ReserveFor(unsigned uiCustomer, unsigned uiSeconds);
  //! zruen rezervace
  /*!
  Zruen rezervace sedadla.
  @param[in] uiCustomer jedinen ID zkaznka kter mus bt shodn s ID uvedenm pi rezervaci
  @return @e true pi spchu
  */
  bool Free(unsigned uiCustomer);
  //! dotaz zda je sedadlo rezervovno
  /*!
  @return @e true v ppad platn rezervace, @e false v ppad volnho sedadla
  */
  bool Reserved(void);
  //! zloha
  /*!
  Vytvoen zlohy tdy vech datovch poloek do proudu (souboru, soketu, ...).
  @param[out] cStream vstupn proud oteven v binrnm mdu
  */
  void Backup(std::ostream *cStream) const;
private:
  bool Reserved(time_t tNow);
  bool m_fReserved;
  time_t m_tExpiry;
  unsigned m_uiCustomer;
};

#endif
