#include <winsock2.h>
#include <string>
#include <iostream>
#include <fstream>
#include <vector>
#include <algorithm>
#include <crtdbg.h>

#pragma hdrstop

#include "Database/MemoryDb.h"
#include "Database/Journal.h"
#include "ServerQuery.h"
#include "ServerConn.h"
#include "ServerTcp.h"
#include "ServerMsg.h"
#include "ServerApp.h"

bool ReadDataDir(std::string& strConf, std::string& strDataDir)
{
  bool fReturn = true;
  std::ifstream cInput(strConf);
  if(cInput.is_open()) {
    std::string strLine;
    while(getline(cInput, strLine) && fReturn) {
      if(strLine[0] == '#')
        continue;
      if(strLine.find("datadir") == 0) {
        auto i = strLine.find("=");
        if(i != std::string::npos)
          strDataDir = strLine.substr(i + 1);
        break;
      }
      else if(strLine.find("msg") == 0 || strLine.find("tcp") == 0) {
        continue;
      }
      else {
        std::cout << "Wrong format of brs_server.conf file." << std::endl;
        fReturn = false;
      }
    }
    cInput.close();
    if(cInput.fail() && !cInput.eof()) {
      std::cout << "Error reading brs_server.conf file." << std::endl;
      fReturn = false;
    }
  }
  else {
    std::cout << "Cannot open brs_server.conf file." << std::endl;
    fReturn = false;
  }
  return fReturn;
}

bool ReadConf(std::string& strConf, std::vector<CServerConnection *>* cServers, CServerQuery *cServerQuery)
{
  bool fReturn = true;
  std::string strData;
  std::ifstream cInput(strConf);
  if(cInput.is_open()) {
    std::string strLine;
    while(getline(cInput, strLine) && fReturn) {
      if(strLine[0] == '#')
        continue;
      if(strLine.find("tcp") == 0) {
        auto i = strLine.find("=");
        if(i != std::string::npos) {
          strData = strLine.substr(i + 1);
          cServers->push_back(new CServerTcp(20, strData.c_str(), CServerQuery::ServerCall, cServerQuery));
        }
      }
      else if(strLine.find("msg") == 0) {
        auto i = strLine.find("=");
        if(i != std::string::npos) {
          strData = strLine.substr(i + 1);
          cServers->push_back(new CServerMsg(strData.c_str(), 20, CServerQuery::ServerCall, cServerQuery));
        }
      }
      else if(strLine.find("datadir") == 0) {
        continue;
      }
      else {
        std::cout << "Wrong format of 'brs_server.conf' file." << std::endl;
        fReturn = false;
      }
    }
    cInput.close();
    if(cInput.fail() && !cInput.eof()) {
      std::cout << "Error reading 'brs_server.conf' file." << std::endl;
      fReturn = false;
    }
  }
  else {
    std::cout << "Cannot open 'brs_server.conf' file." << std::endl;
    fReturn = false;
  }
  return fReturn;
}


int main(int argc, char **argv)
{
#ifdef _DEBUG
	_CrtSetDbgFlag( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );
#endif


  bool fRestore = false;
  std::string strPath(argv[0]);
  std::string strConf = strPath.substr(0, strPath.find_last_of("/\\") + 1);
  strConf += "brs_server.conf";

  std::cout << "Bus Reservation System 1.0 - Server\nPetr Stefan - Programming in C++, 2013/2014" << std::endl << std::endl;

  CMemoryDb *cDb;

  std::string strDataDir;
  bool fResult = ReadDataDir(strConf, strDataDir);
  if(!fResult)
    return 0;

  if(argc > 1) { // obnova dat z uren zlohy
    std::ifstream cFile;
    cFile.open(argv[1], std::ifstream::binary);
    if(cFile.is_open()) {
      cDb = new CMemoryDb(&cFile);
      cFile.close();
      std::cout << "Database restored." << std::endl;
      fRestore = true;
    }
    else {
      cDb = new CMemoryDb();
      std::cout << "Restore database failed. Creating empty one." << std::endl;
    }
  }
  else { // naten posledn platn databze
    std::ifstream cFile;
    cFile.open(strDataDir + InternalBackupName(), std::ifstream::binary);
    if(cFile.is_open()) {
      cDb = new CMemoryDb(&cFile);
      cFile.close();
      std::cout << "Database loaded." << std::endl;
      fRestore = true;
    }
    else
      cDb = new CMemoryDb();
    // zpracovn urnlu (pokud existuje)
    CReadJournal cJournal(strDataDir.c_str(), cDb);
    unsigned uiJournalItems;
    if(cJournal.Replay(&uiJournalItems)) {
      if(uiJournalItems != 0) {
        std::cout << "Journal - " << uiJournalItems << " items processed." << std::endl;
        fRestore = true;
      }
    }
    else
      std::cout << "Processing journal failed" << " (at item " << uiJournalItems << ")" << std::endl;
  }
  cDb->SetDataDir(strDataDir.c_str());

  CServerQuery *cServerQuery = new CServerQuery(cDb);

  std::vector<CServerConnection *> cServers;
  if(!ReadConf(strConf, &cServers, cServerQuery)) {
    while(!cServers.empty()) {
      delete cServers.back();
      cServers.pop_back();
    }
    char cChar;
    std::cin.get(cChar);
    return 0;
  }
  
  CServerApp cTerminal(&cServers, cDb);
  cTerminal.Run(fRestore);

  std::for_each(cServers.begin(), cServers.end(), [](CServerConnection* cServ){delete cServ;});

  delete cServerQuery;
  delete cDb;

  return 0;
}
