#include <string>
#include <sstream>
#include <iostream>
#include <ctype.h>
#include <algorithm>
#include <fstream>

#pragma hdrstop

#include "ServerApp.h"

CServerApp::CServerApp(std::vector<CServerConnection *>* cServers, CMemoryDb* cDb) : m_cServers(cServers), m_cDb(cDb), m_fRunning(true)
{
  m_cCommands.reserve(5);
  m_cCommands.insert(std::make_pair("backup", &CServerApp::Backup));
  m_cCommands.insert(std::make_pair("info", &CServerApp::Info));
  m_cCommands.insert(std::make_pair("start", &CServerApp::Start));
  m_cCommands.insert(std::make_pair("stop", &CServerApp::Stop));
  m_cCommands.insert(std::make_pair("help", &CServerApp::Help));
}

CServerApp::~CServerApp()
{
}

bool CServerApp::Run(bool fInfo)
{
  Start();
  if(fInfo)
    Info();
  while(true) {
    // nati dek ze vstupu
    while(m_strInput.empty())
      if(!Read(m_strInput)) {
        Stop();
        return true; // nap. pi EOF ze souboru pesmrovanho na vstup
      }
    std::string strComm = GetOneParam(); // vyzvednut pkazu
    std::transform(strComm.begin(), strComm.end(), strComm.begin(), tolower); // pevod na mal psmena
    if(strComm == "exit")
      break;
    else {
      auto i = m_cCommands.find(strComm);
      if(i == m_cCommands.end())
        std::cout << "Unknown command." << std::endl;
      else
        (this->*(i->second))();
    }
    m_strInput.clear();
  }
  Stop();
  return true;
}

std::string CServerApp::GetOneParam(void)
{
  std::istringstream cInput(m_strInput);
  std::string strParam;
  cInput >> strParam;
  if(strParam[0] == '"' && strParam.length() > 1) {
    strParam = strParam.substr(1);
    while(strParam[strParam.length() - 1] != '"') {
      std::string strTemp;
      cInput >> strTemp;
      strParam += " ";
      strParam += strTemp;
    }
    strParam = strParam.substr(0, strParam.length() - 1);
    m_strInput = m_strInput.substr(m_strInput.find(strParam) + strParam.length()+1);
  }
  else
    m_strInput = m_strInput.substr(m_strInput.find(strParam) + strParam.length());
  return strParam;
}

void CServerApp::Backup(void)
{
  std::string strPath = GetOneParam();
  if(strPath.empty()) {
    std::cout << "Write backup file name." << std::endl;
    return;
  }
  std::ofstream cFile;
  cFile.open(strPath, std::ofstream::binary | std::ofstream::trunc);
  if(cFile.is_open()) {
    if(!m_cDb->Backup(&cFile)) {
      std::cout << "Backup failed." << std::endl;
    }
    else
      std::cout << "Backup succeeded - written " << cFile.tellp() << " B" << std::endl;
    cFile.close();
  }
  else {
    std::cout << "Failed open output file." << std::endl;
  }
}

void CServerApp::Info(void)
{
  if(m_fRunning)
    std::cout << "Communication servers are running. (total: " << m_cServers->size() << ")" << std::endl;
  else
    std::cout << "Communication servers are stopped. (total: " << m_cServers->size() << ")" << std::endl;
  RDbInfo rInfo;
  m_cDb->Info(&rInfo);
  std::cout << "Cities:   " << rInfo.uiCities << std::endl <<
    "Aliases:  " << rInfo.uiCityAliases << std::endl <<
    "Plans:    " << rInfo.uiPlans << std::endl <<
    "Buses:    " << rInfo.uiBuses << std::endl;

}

void CServerApp::Start(void)
{
  m_cDb->OnStart();
  std::for_each(m_cServers->begin(), m_cServers->end(), [](CServerConnection* cServ){cServ->Start();});
  m_fRunning = true;
  std::cout << "Servers started successfully." << std::endl;
}

void CServerApp::Stop(void)
{
  std::for_each(m_cServers->begin(), m_cServers->end(), [](CServerConnection* cServ){cServ->Stop();});
  m_fRunning = false;
  m_cDb->OnStop();
  std::cout << "Servers stopped successfully." << std::endl;
}

void CServerApp::Help(void)
{
  std::cout << "List of available commands:" << std::endl;
  std::cout << "EXIT\tcloses the program" << std::endl;
  std::cout << "BACKUP\tbackups database to a file\n\tExample:\tbackup database.bac" << std::endl;
  std::cout << "START\tstarts connection servers" << std::endl;
  std::cout << "STOP\tstops connection servers" << std::endl;
  std::cout << "INFO\twrites some information about servers and database" << std::endl;
  std::cout << "HELP\tprints this help page" << std::endl;
  std::cout << "\nCommands are case insensitive." << std::endl;
}

bool CServerApp::Read(std::string &strBuffer)
{
  std::cout << "> ";
  getline(std::cin, strBuffer);
  return !std::cin.fail();
}
