#ifndef _SERVERAPP_H
#define _SERVERAPP_H

#include <string>
#include <unordered_map>

#include "Database/MemoryDb.h"
#include "ServerQuery.h"
#include "ServerConn.h"

/*!
@file ServerApp.h
Deklarace sprvcovskho terminlu @ref CServerApp.
*/

class CServerApp;
//! typ metody pro zpracovn pkazu
/*!
Jednotliv metody jsou zpracovvny metodami tdy @ref CClientApp. Penos daj 
se dje pomoc lenskch promnnch tdy, proto metody nemaj dn parametry a 
nvratov hodnota je typu @c void.
*/
typedef void (CServerApp::*FnCommand)(void);

//! sprvcovsk terminl
/*!
Tda obsluhuje konzoli serverov aplikace. Pomoc n lze provdt mj. sprvu 
databze pes rozhran @ref CAdmin a zen modulu pro meziprocesovou komunikaci. 
*/
class CServerApp {
public:
  //! konstruktor
  /*!
  @param[in] cServers kontejner s ukazateli na komunikan servery
  @param[in] cDb ukazatel na databzi
  */
  CServerApp(std::vector<CServerConnection *>* cServers, CMemoryDb* cDb);
  //! destruktor
  ~CServerApp();
  //! sputn terminlu
  /*!
  Metoda pro sputn sprvcovskho terminlu. Komunikan servery se spust ji pi
  svm vytvoen.
  @param[in] fInfo infikuje, zda se m po startu vypsat Info
  */
  bool Run(bool fInfo);
private:
  std::string GetOneParam(void);
  std::unordered_map<std::string, FnCommand> m_cCommands;
  std::string m_strInput;
  std::vector<CServerConnection *>* m_cServers;
  CMemoryDb* m_cDb;
  bool m_fRunning;
  void Backup(void);
  void Info(void);
  void Start(void);
  void Stop(void);
  void Help(void);
  bool Read(std::string &strBuffer);
};

#endif
