#ifndef _SERVERCONN_H
#define _SERVERCONN_H

#include "../Common/UtilClass.h"

/*!
@file ServerConn.h
Deklarace tdy @ref CServerConnection a typu uivatelsk funkce @ref FnProcess.
*/

//! typ funkce pro zpracovn dat
/*!
@param[in] pParam uivatelsk parametr pro libovoln pouit
@param[in, out] cCmd instance tdy @ref CBlob s binrnmi daty:
@li na vstupu pkaz k vykonn na serveru
@li po nvratu data po zpracovn
@return informace o proveden funkce, kter se pen pomoc stavovho slova zpt ke klientovi
*/
typedef unsigned (*FnProcess)(void *pParam, CBlob *cCmd);

//! serverov st komunikace
/*!
Tda slou k deklaraci spolench metod komunikanho rozhran na stran serveru. Konkrtn
specializace pro kad zpsob transportu se provd v potomcch. Veejn metody umouj pouze
sputn a zastaven serveru (tj. pepnut do aktivnho stavu a zpt).
*/
class CServerConnection {
public:
  //! konstruktor
  /*!
  K zpracovn pkazu je piazena uivatelsk funkce a parametr pro libovoln
  pouit pi jejm voln.
  @param[in] fnProcess adresa uivatelsk funkce typu @c FnProcess
  @param[in] pParam parametr pro libovoln pouit (po petypovn)
  @n Me to bt napklad ukazatel na virtuln tabulku tdy, jej je funkce statickou metodou.
  */
  CServerConnection(FnProcess fnProcess, void *pParam);
  //! destruktor
  virtual ~CServerConnection();
  //! sputn serveru
  /*! @return spch operace (@e true, pokud server peel do aktivnho stavu) */
  virtual bool Start(void) = 0;
  //! zastaven serveru
  /*! Server pestane pijmat zprvy od kompatibilnch klient. */
  virtual void Stop(void) = 0;
protected:
  //! vyvoln uivatelsk funkce
  /*!
  Po pjmu pkazu je pomoc voln metody testovno, zda je definovna uivatelsk funkce vkonnho
  modulu serveru a v kladnm ppad je tato zavolna a pedny j parametry z pijatho pkazu.
  Vstupn parametry jsou pak pedny k dalmu zpracovn komunikan tdou serveru, tj. k odesln
  zpt klientovi.
  @param[out] puiResult nvratov hodnota uivatelsk funkce
  @param[in, out] cCmd vstupn i vstupn binrn data pro voln uivatelsk funkce
  @return spch vyvoln uivatelsk funkce (@e false, pokud funkce nen v konstruktoru piazena)
  */
  bool CallProcessing(unsigned *puiResult, CBlob *cCmd);
private:
  FnProcess m_fnProcess;
  void *m_pParam;
};

#endif
