#ifndef _SERVERHTTP_H
#define _SERVERHTTP_H

#include "ServerTcp.h"

/*!
@file ServerHttp.h
Deklarace tdy @ref CServerHttp.
*/

//! serverov st pro komunikaci s HTTP klientem
/*!
Jednoduch vcevlknov server HTTP pebr vekerou funkcionalitu TCP serveru, jedin a zsadn
odlinost je v prci s pijatmi daty. Zatmco TCP server m na druh stran kompatibilnho klienta,
proti tomuto serveru stoj webov prohle. Server mus pro klienta pipravit vhodnou webovou
strnku s formulem, z nho pak me extrahovat potebn data, kter pak po zpracovn opt ve
form HTML kdu prezentuje klientovi. V tomto smyslu pebr do znan mry lohu vkonnho modulu
na klientsk stran.
*/
class CServerHttp : public CServerTcp {
public:
  //! konstruktor
  /*!
  Vytvoen instance vicevlknovho HTTP serveru.
  @param[in] uiThreads maximln poet paraleln bcch zpracovatelskch vlken
  @param[in] pszHost uren IP adresy a portu pro naslouchac soket
  @n U serveru je mon naslouchat na vech IP adresch, kter m pota pidleny a pijmat
  tak zprvy od vech vzdlench i loklnch klient. Podporovan formty jsou popsny u
  @ref CSocketEndpoint::CSocketEndpoint.
  @param[in] fnProcess adresa funkce typu @ref FnProcess pro zpracovn pkaz na serveru
  @param[in] pParam libovoln parametr, kter bude pedn uivatelsk funkci
  */
  CServerHttp(unsigned uiThreads, const char *pszHost, FnProcess fnProcess, void *pParam);
  //! destruktor
  /*! Zajiuje dealokaci vech vlastnch zdroj, ovem a po dokonen vech zpracovatelskch vlken. */
  virtual ~CServerHttp();
protected:
  //! komunikace a zpracovn pkazu
  /*!
  Nov implementace metody vlastn zajiuje vekerou novou (tj. odlinou) funkcionalitu tdy.
  Je zodpovdn za nejzkladnj prci s HTTP, ppravu webovch strnek s formuli, zpracovn
  zaslanch dat atd.
  @param[in] hClient soket, na kterm je navzno spojen s klientem (prohleem)
  */
  virtual void CmdProcessing(SOCKET hClient);
};

#endif
