#include <windows.h>
#include <process.h>

#pragma hdrstop

#include "ServerMsg.h"
#include "ServerConn.h"
#include "../Common/MsgEndpoint.h"
#include "../Common/Const.h"
#include "../Common/Command.h"
#include "../Common/UtilClass.h"

const unsigned MIN_THREADS = 1;
const unsigned MAX_THREADS = 10;

CServerMsg::CServerMsg(const char *pszServerWnd, unsigned uiThreads, FnProcess fnProcess, void *pParam) :
  CServerConnection(fnProcess, pParam), 
  CMsgEndpoint(pszServerWnd, MSG_SERVER_WND_CLASS)
{
  Enable(false);
  m_hThread = NULL;
  m_hTerminateEvent = CreateEvent(NULL, TRUE, FALSE, NULL); // nesignalizovan manual-reset udlost
  if(uiThreads < MIN_THREADS)
    uiThreads = MIN_THREADS;
  else if(uiThreads > MAX_THREADS)
    uiThreads = MAX_THREADS;
  m_hThreadCounter = CreateSemaphore(NULL, uiThreads, uiThreads, NULL); // potn zpracovatelskch vlken
  m_uiThreads = uiThreads;
}

CServerMsg::~CServerMsg()
{
  if(m_hThread != NULL)
    Stop(); // zastaven bcho serveru
  CloseHandle(m_hTerminateEvent);
  CloseHandle(m_hThreadCounter);
}

bool CServerMsg::Start(void)
{
  if(m_hThread == NULL) {
    ClearReceived();
    ResetEvent(m_hTerminateEvent); // umonn innosti pracovnho vlkna
    m_hThread = (HANDLE)_beginthreadex(NULL, 0, WorkThread, this, 0, NULL); // odstartuj pracovn vlkno serveru
    return true;
  }
  return false;
}

void CServerMsg::Stop(void)
{
  if(m_hThread != NULL) {
    SetEvent(m_hTerminateEvent);
    WaitForSingleObject(m_hThread, INFINITE); // ekn na ukonen pracovnho vlkna serveru
    CloseHandle(m_hThread);
    WaitForThreads(); // pokn na dokonen vech zpracovatelskch vlken
    m_hThread = NULL;
  }
}

// pracovn vlkno serveru
unsigned int WINAPI CServerMsg::WorkThread(LPVOID pThreadParams)
{
  CServerMsg *cThis;
  HANDLE ahEvent[2];
  HANDLE hThread;

  cThis = (CServerMsg *)pThreadParams;
  cThis->Enable(true); // povolen pjmu zprv
  ahEvent[0] = cThis->m_hTerminateEvent;
  ahEvent[1] = cThis->ReceivedEvent();
  while(true) {
    switch(WaitForMultipleObjects(2, ahEvent, FALSE, INFINITE)) {
      case WAIT_OBJECT_0 + 0: // poadavek na ukonen
        cThis->Enable(false); // ukonen pjmu zprv
        return 0; 
      case WAIT_OBJECT_0 + 1: // pijata data
        if(WaitForSingleObject(cThis->m_hThreadCounter, 100) == WAIT_OBJECT_0) { // je poet vlken vyerpn?
          hThread = (HANDLE)_beginthreadex(NULL, 0, CmdThread, cThis, 0, NULL); // zpracovn dat ve zvltnm vlkn
          CloseHandle(hThread);
        }
        else
          SetEvent(ahEvent[1]); // data dosud nebyla zpracovna
        break;
    }
  }
}

// vlkno pro zpracovn pkazu
unsigned int WINAPI CServerMsg::CmdThread(LPVOID pThreadParams)
{
  CServerMsg *cThis;
  bool fResult;
  unsigned uiResult;
  RMsgData rMsgData;
  CMsgData *cCmd;

  cThis = (CServerMsg *)pThreadParams;
  cCmd = new CMsgData();
  // vyzvednut pkazu
  if(cThis->Get(cCmd)) {
    rMsgData = cCmd->Get();
    // zavolej zpracovn pkazu
    fResult = cThis->CallProcessing(&uiResult, cCmd);
    // odesln odpovdi
    rMsgData.m_dwId = CMD_SETFUNCRESULT(uiResult) + (rMsgData.m_dwId & CMD_COUNTER_MASK);
    if(!fResult) {
      rMsgData.m_dwId |= CMD_FUNCFAIL_FLAG;
      cCmd->Clear();
    }
    cCmd->Set(rMsgData);
    cThis->Send(cCmd);
  }
  delete cCmd;
  // vlkno kon, aktualizuj potadlo
  ReleaseSemaphore(cThis->m_hThreadCounter, 1, NULL);
  return 0;
}

void CServerMsg::WaitForThreads(void)
{
  unsigned uiLastCount;
  unsigned uiFinish;

  uiFinish = m_uiThreads - 1;
  // ekn na dokonen vech vlken zpracovn pkaz
  while(true) {
    WaitForSingleObject(m_hThreadCounter, INFINITE); // -1
    ReleaseSemaphore(m_hThreadCounter, 1, (LPLONG)&uiLastCount); // +1
    if(uiLastCount == uiFinish)
      return;
    Sleep(100);
  }
}
