#ifndef _SERVERMSG_H
#define _SERVERMSG_H

#include "ServerConn.h"
#include "../Common/MsgEndpoint.h"

/*!
@file ServerMsg.h
Deklarace tdy @ref CServerMsg.
*/

//! serverov st komunikace pomoc zprv Windows
/*!
Tda zabezpeuje serverovou st komunikace pomoc zprv operanho systmu. Implementuje
rozhran @ref CServerConnection, pro vytvoen a obsluhu pijmacho okna pouv funcionalitu
tdy @ref CMsgEndpoint.

Pomocn vlkno pro obsluhu okna je vytvoeno po celou dobu ivota instance serveru, je vytvoeno 
v konstruktoru @ref CMsgEndpoint.
Volnm @c Start se vytv pracovn vlkno a server pechz do aktivnho stavu. Pracovn vlkno 
ek na udlost signalizujc pchod zprvy. Pchoz zprva @c WM_COPYDATA je zpracovna procedurou 
okna a uloena do internho kontajneru. Pracovn vlkno nsledn vytv zpracovatelsk vlkno a ek
na pchod dal zprvy. Zpracovatelsk vlkno vyzvedne zprvu z kontajneru a zavol uivatelem
definovou funkci pro zpracovn dat. Nakonec odele vsledn data formou zprvy typu @c WM_COPYDATA 
pomoc funkce @c SendMessage do okna, jeho handle byl uveden v pijatch datech, a svou innost 
ukon. Mezitm pracovn vlkno mohlo vytvoit dal zpracovatelsk vlkna, kterch tak me 
existovat vce paraleln.
@image html ServerMsg.png "Vlkna pi innosti serveru"
Vlkno, kter vytv instanci serveru, je na diagramu znzornno lutmi bloky, mode je vyznaeno
pracovn vlkno serveru a oranov zpracovatelsk vlkno (jeden zstupce). Funcionalita vytvoen
okna, obsluha zprv v procedue okna a obsluha smyky zprv je zddna od @ref CMsgEndpoint, na
diagramu pouv barvu edou.

Nepedpokld se vytven vce instanc tohoto typu serveru, proto se pro jmno a tdu internho okna
implicitn pouij defaultn hodnoty z @ref Const.h (co nsledn umouje klientm toto okno 
v systmu vyhledat pomoc @c FindWindowEx).
*/
class CServerMsg : public CServerConnection, private CMsgEndpoint {
public:
  //! konstruktor
  /*!
  Vytvoen instance vcevlknovho serveru pro pjem a zpracovn pkaz transportovanch pomoc 
  zprv Windows. Pro tdu internho okna vyuv defaultn hodnoty.
  @param[in] pszServerWnd jmno skrytho okna, kter slou pro pjem zprv od klient
  @param[in] uiThreads maximln poet paraleln bcch zpracovatelskch vlken
  @param[in] fnProcess adresa funkce typu @ref FnProcess pro zpracovn pkaz na serveru
  @param[in] pParam libovoln parametr, kter bude pedn uivatelsk funkci
  */
  CServerMsg(const char *pszServerWnd, unsigned uiThreads, FnProcess fnProcess, void *pParam);
  //! destruktor
  /*!
  Provede zruen internho pijmacho okna, ek na dokonen vech zpracovatelskch vlken a 
  nakonec odele zprvu @c WM_QUIT, m ukon smyku zprv vlkna obsluhy okna, kter nsledn tak 
  kon svou innost. Nakonec uvoln veker alokovan zdroje.
  */
  virtual ~CServerMsg();
  virtual bool Start(void);
  virtual void Stop(void);
private:
  unsigned m_uiThreads;
  HANDLE m_hThread;
  HANDLE m_hTerminateEvent;
  HANDLE m_hThreadCounter;
  static unsigned int WINAPI WorkThread(LPVOID pThreadParams);
  static unsigned int WINAPI CmdThread(LPVOID pThreadParams);
  void WaitForThreads(void);
};

#endif
