#define _CRT_SECURE_NO_WARNINGS

#include <list>
#include <time.h>

#pragma hdrstop

#include "ServerQuery.h"
#include "../Common/UtilClass.h"
#include "../Common/Query.h"
#include "../Common/Command.h"

CServerQuery::CServerQuery(CQuery *cQuery)
{
  m_cQuery = cQuery;
}

CServerQuery::~CServerQuery()
{
}

static void FnAddCity(CQuery *cQuery, CBlob *cBlob)
{
  unsigned uiResult = cQuery->AddCity((const char *)cBlob->DataPointer() + sizeof(unsigned));
  cBlob->Set(&uiResult, sizeof(unsigned));
}

static void FnAddCityAlias(CQuery *cQuery, CBlob *cBlob)
{
  const char *pszName = (const char *)cBlob->DataPointer() + sizeof(unsigned);
  unsigned uiCity;
  cBlob->CopyData(&uiCity, sizeof(unsigned), sizeof(unsigned) + static_cast<unsigned>(strlen(pszName) + 1));
  unsigned uiResult = cQuery->AddCityAlias(pszName, uiCity) ? 1 : 0;
  cBlob->Set(&uiResult, sizeof(unsigned));
}

static void FnDeleteCity(CQuery *cQuery, CBlob *cBlob)
{
  unsigned uiResult = cQuery->DeleteCity((const char *)cBlob->DataPointer() + sizeof(unsigned)) ? 1 : 0;
  cBlob->Set(&uiResult, sizeof(unsigned));
}

static void FnFindCity(CQuery *cQuery, CBlob *cBlob)
{
  unsigned uiResult = cQuery->FindCity((const char *)cBlob->DataPointer() + sizeof(unsigned));
  cBlob->Set(&uiResult, sizeof(unsigned));
}

static void FnCityInfo(CQuery *cQuery, CBlob *cBlob)
{
  unsigned uiCity;
  unsigned uiLinesOut;
  unsigned uiLinesIn;
  RCityInfo rInfo;
  cBlob->CopyData(&uiCity, sizeof(unsigned), sizeof(unsigned));
  unsigned uiResult = cQuery->CityInfo(uiCity, &rInfo, &uiLinesOut, &uiLinesIn) ? 1 : 0;
  unsigned uiNameLength = static_cast<unsigned>(rInfo.strName.length());
  // dlka = vsledek + id + jmno + odjdjc + pijdjc
  unsigned uiLength = sizeof(unsigned) + sizeof(unsigned) + uiNameLength + 1 + sizeof(unsigned) + sizeof(unsigned);
  char *pcData = new char [uiLength];
  // nvratov hodnota
  memcpy(pcData, &uiResult, sizeof(unsigned));
  unsigned uiOffset = sizeof(unsigned);
  // vstupn parametry
  memcpy(pcData + uiOffset, &rInfo.uiId, sizeof(unsigned));
  uiOffset += sizeof(unsigned);
  strcpy(pcData + uiOffset, rInfo.strName.c_str());
  uiOffset += uiNameLength + 1;
  memcpy(pcData + uiOffset, &uiLinesOut, sizeof(unsigned));
  uiOffset += sizeof(unsigned);
  memcpy(pcData + uiOffset, &uiLinesIn, sizeof(unsigned));
  cBlob->Set(pcData, uiLength);
  delete [] pcData;
}

static void FnAddBusPlan(CQuery *cQuery, CBlob *cBlob)
{
  // vstupn parametry
  const char *pszName = (const char *)cBlob->DataPointer() + sizeof(unsigned);
  unsigned uiOffset = sizeof(unsigned) + static_cast<unsigned>(strlen(pszName) + 1);
  unsigned uiSeats;
  cBlob->CopyData(&uiSeats, sizeof(unsigned), uiOffset);
  uiOffset += sizeof(unsigned);
  const char *pszTextPlan = (const char *)cBlob->DataPointer() + uiOffset;
  uiOffset += static_cast<unsigned>(strlen(pszTextPlan)) + 1;
  unsigned uiBinSize;
  cBlob->CopyData(&uiBinSize, sizeof(unsigned), uiOffset);
  uiOffset += sizeof(unsigned);
  CBlobEx<std::string> cBlobEx(pszTextPlan, (const char *)cBlob->DataPointer() + uiOffset, uiBinSize);
  unsigned uiResult = cQuery->AddBusPlan(pszName, uiSeats, &cBlobEx);
  // nvratov hodnota
  cBlob->Set(&uiResult, sizeof(unsigned));
}

static void FnDeleteBusPlan(CQuery *cQuery, CBlob *cBlob)
{
  unsigned uiResult = cQuery->DeleteBusPlan((const char *)cBlob->DataPointer() + sizeof(unsigned)) ? 1 : 0;
  cBlob->Set(&uiResult, sizeof(unsigned));
}

static void FnFindBusPlan(CQuery *cQuery, CBlob *cBlob)
{
  unsigned uiResult = cQuery->FindBusPlan((const char *)cBlob->DataPointer() + sizeof(unsigned));
  cBlob->Set(&uiResult, sizeof(unsigned));
}

static void FnBusPlanInfo(CQuery *cQuery, CBlob *cBlob)
{
  unsigned uiPlan;
  unsigned uiBuses;
  RPlanInfo rInfo;
  CBlobEx<std::string> cBlobEx;
  cBlob->CopyData(&uiPlan, sizeof(unsigned), sizeof(unsigned));
  unsigned uiResult = cQuery->BusPlanInfo(uiPlan, &rInfo, &uiBuses, &cBlobEx) ? 1 : 0;
  unsigned uiNameLength = static_cast<unsigned>(rInfo.strName.length());
  unsigned uiPlanStrLength = static_cast<unsigned>(cBlobEx.Get().length());
  unsigned uiPlanBinSize = cBlobEx.DataLen();
  // dlka = vsledek + id + jmno + sedadel + autobus + textov pln + dlka binrnch dat + data
  unsigned uiLength = sizeof(unsigned) + sizeof(unsigned) + uiNameLength + 1 + sizeof(unsigned) + sizeof(unsigned) + uiPlanStrLength + 1 + sizeof(unsigned) + uiPlanBinSize;
  char *pcData = new char [uiLength];
  // nvratov hodnota
  memcpy(pcData, &uiResult, sizeof(unsigned));
  unsigned uiOffset = sizeof(unsigned);
  // vstupn parametry
  memcpy(pcData + uiOffset, &rInfo.uiId, sizeof(unsigned));
  uiOffset += sizeof(unsigned);
  strcpy(pcData + uiOffset, rInfo.strName.c_str());
  uiOffset += uiNameLength + 1;
  memcpy(pcData + uiOffset, &rInfo.uiSeats, sizeof(unsigned));
  uiOffset += sizeof(unsigned);
  memcpy(pcData + uiOffset, &uiBuses, sizeof(unsigned));
  uiOffset += sizeof(unsigned);
  strcpy(pcData + uiOffset, cBlobEx.Get().c_str());
  uiOffset += uiPlanStrLength + 1;
  memcpy(pcData + uiOffset, &uiPlanBinSize, sizeof(unsigned));
  uiOffset += sizeof(unsigned);
  memcpy(pcData + uiOffset, cBlobEx.DataPointer(), uiPlanBinSize);
  cBlob->Set(pcData, uiLength);
  delete [] pcData;
}

static void FnAddBus(CQuery *cQuery, CBlob *cBlob)
{
  RBusInfo rInfo;
  cBlob->CopyData(&rInfo, sizeof(RBusInfo), sizeof(unsigned));
  unsigned uiResult = cQuery->AddBus(&rInfo);
  cBlob->Set(&uiResult, sizeof(unsigned));
}

static void FnDeleteBus(CQuery *cQuery, CBlob *cBlob)
{
  unsigned uiBus;
  cBlob->CopyData(&uiBus, sizeof(unsigned), sizeof(unsigned));
  unsigned uiResult = cQuery->DeleteBus(uiBus) ? 1 : 0;
  cBlob->Set(&uiResult, sizeof(unsigned));
}

static void FnFindBus(CQuery *cQuery, CBlob *cBlob)
{
  unsigned uiCityFrom;
  unsigned uiCityTo;
  time_t tLeaveTime;
  cBlob->CopyData(&uiCityFrom, sizeof(unsigned), sizeof(unsigned));
  cBlob->CopyData(&uiCityTo, sizeof(unsigned), sizeof(unsigned) + sizeof(unsigned));
  cBlob->CopyData(&tLeaveTime, sizeof(time_t), sizeof(unsigned) + sizeof(unsigned) + sizeof(unsigned));
  unsigned uiResult = cQuery->FindBus(uiCityFrom, uiCityTo, tLeaveTime);
  cBlob->Set(&uiResult, sizeof(unsigned));
}

static void FnBusInfo(CQuery *cQuery, CBlob *cBlob)
{
  unsigned uiBus;
  cBlob->CopyData(&uiBus, sizeof(unsigned), sizeof(unsigned));
  RBusInfo rInfo;
  bool fEnabled;
  unsigned uiSeats;
  std::list<unsigned> cSeats;
  unsigned uiResult = cQuery->BusInfo(uiBus, &rInfo, &fEnabled, &uiSeats, &cSeats) ? 1 : 0;
  unsigned uiListCount = static_cast<unsigned>(cSeats.size());
  // dlka = vsledek + bus info + povolen + sedadel + poet volnch sedadel + voln sedadla
  unsigned uiLength = sizeof(unsigned) + sizeof(RBusInfo) + sizeof(unsigned) + sizeof(unsigned) + sizeof(unsigned) + uiListCount * sizeof(unsigned);
  char *pcData = new char [uiLength];
  // nvratov hodnota
  memcpy(pcData, &uiResult, sizeof(unsigned));
  unsigned uiOffset = sizeof(unsigned);
  // vstupn parametry
  memcpy(pcData + uiOffset, &rInfo, sizeof(RBusInfo));
  uiOffset += sizeof(RBusInfo);
  unsigned uiEnabled = fEnabled ? 1 : 0;
  memcpy(pcData + uiOffset, &uiEnabled, sizeof(unsigned));
  uiOffset += sizeof(unsigned);
  memcpy(pcData + uiOffset, &uiSeats, sizeof(unsigned));
  uiOffset += sizeof(unsigned);
  memcpy(pcData + uiOffset, &uiListCount, sizeof(unsigned)); // poet poloek
  uiOffset += sizeof(unsigned);
  for(std::list<unsigned>::iterator i = cSeats.begin(); i != cSeats.end(); ++i) {
    unsigned uiValue = *i;
    memcpy(pcData + uiOffset, &uiValue, sizeof(unsigned)); // jednotliv hodnoty
    uiOffset += sizeof(unsigned);
  }
  cBlob->Set(pcData, uiLength);
  delete [] pcData;
}

static void FnEnableBus(CQuery *cQuery, CBlob *cBlob)
{
  unsigned uiBus;
  cBlob->CopyData(&uiBus, sizeof(unsigned), sizeof(unsigned));
  unsigned uiResult = cQuery->EnableBus(uiBus) ? 1 : 0;
  cBlob->Set(&uiResult, sizeof(unsigned));
}

static void FnDisableBus(CQuery *cQuery, CBlob *cBlob)
{
  unsigned uiBus;
  cBlob->CopyData(&uiBus, sizeof(unsigned), sizeof(unsigned));
  unsigned uiResult = cQuery->DisableBus(uiBus) ? 1 : 0;
  cBlob->Set(&uiResult, sizeof(unsigned));
}

static void FnNewCustomer(CQuery *cQuery, CBlob *cBlob)
{
  unsigned uiCustomer = cQuery->NewCustomer();
  cBlob->Set(&uiCustomer, sizeof(unsigned));
}

static void SeatOperationInput(const CBlob *cBlob, unsigned *puiBus, std::list<unsigned> *cSeats, unsigned *puiCustomer)
{
  unsigned uiOffset = sizeof(unsigned);
  // slo autobusu
  cBlob->CopyData(puiBus, sizeof(unsigned), uiOffset);
  uiOffset += sizeof(unsigned);
  // seznam sedadel - poet, hodnoty (pedpokld se przdn seznam pi voln funkce)
  unsigned uiItems;
  unsigned uiValue;
  cBlob->CopyData(&uiItems, sizeof(unsigned), uiOffset);
  uiOffset += sizeof(unsigned);
  while(uiItems-- != 0) {
    cBlob->CopyData(&uiValue, sizeof(unsigned), uiOffset);
    uiOffset += sizeof(unsigned);
    cSeats->push_back(uiValue);
  }
  // zkaznk
  cBlob->CopyData(puiCustomer, sizeof(unsigned), uiOffset);
}

static void SeatOperationOutput(CBlob *cBlob, unsigned uiResult, const std::list<unsigned> *cSeats)
{
  unsigned uiListCount = static_cast<unsigned>(cSeats->size());
  // dlka = vsledek + poet sedadel v seznamu + sedadla
  unsigned uiLength = sizeof(unsigned) + sizeof(unsigned) + uiListCount * sizeof(unsigned);
  char *pcData = new char [uiLength];
  // nvratov hodnota
  memcpy(pcData, &uiResult, sizeof(unsigned));
  unsigned uiOffset = sizeof(unsigned);
  // vstupn parametry
  memcpy(pcData + uiOffset, &uiListCount, sizeof(unsigned)); // poet poloek
  uiOffset += sizeof(unsigned);
  for(std::list<unsigned>::const_iterator i = cSeats->begin(); i != cSeats->end(); ++i) {
    unsigned uiValue = *i;
    memcpy(pcData + uiOffset, &uiValue, sizeof(unsigned)); // jednotliv hodnoty
    uiOffset += sizeof(unsigned);
  }
  cBlob->Set(pcData, uiLength);
  delete [] pcData;
}

// operace se sedadly - prce s vstupnmi a vstupnmi parametry je stejn, zabezpeuj ji pomocn funkce

static void FnAskSeats(CQuery *cQuery, CBlob *cBlob)
{
  unsigned uiBus;
  std::list<unsigned> cSeats;
  unsigned uiCustomer;
  SeatOperationInput(cBlob, &uiBus, &cSeats, &uiCustomer);
  unsigned uiResult = cQuery->AskSeats(uiBus, &cSeats, uiCustomer);
  SeatOperationOutput(cBlob, uiResult, &cSeats);
}

static void FnLockSeats(CQuery *cQuery, CBlob *cBlob)
{
  unsigned uiBus;
  std::list<unsigned> cSeats;
  unsigned uiCustomer;
  SeatOperationInput(cBlob, &uiBus, &cSeats, &uiCustomer);
  unsigned uiResult = cQuery->LockSeats(uiBus, &cSeats, uiCustomer);
  SeatOperationOutput(cBlob, uiResult, &cSeats);
}

static void FnUnlockSeats(CQuery *cQuery, CBlob *cBlob)
{
  unsigned uiBus;
  std::list<unsigned> cSeats;
  unsigned uiCustomer;
  SeatOperationInput(cBlob, &uiBus, &cSeats, &uiCustomer);
  unsigned uiResult = cQuery->UnlockSeats(uiBus, &cSeats, uiCustomer);
  SeatOperationOutput(cBlob, uiResult, &cSeats);
}

static void FnNextBus(CQuery *cQuery, CBlob *cBlob)
{
  unsigned uiBus;
  cBlob->CopyData(&uiBus, sizeof(unsigned), sizeof(unsigned));
  unsigned uiResult = cQuery->NextBus(uiBus);
  cBlob->Set(&uiResult, sizeof(unsigned));
}

static void FnNextCity(CQuery *cQuery, CBlob *cBlob)
{
  unsigned uiCity;
  cBlob->CopyData(&uiCity, sizeof(unsigned), sizeof(unsigned));
  unsigned uiResult = cQuery->NextCity(uiCity);
  cBlob->Set(&uiResult, sizeof(unsigned));
}

static void FnNextPlan(CQuery *cQuery, CBlob *cBlob)
{
  unsigned uiPlan;
  cBlob->CopyData(&uiPlan, sizeof(unsigned), sizeof(unsigned));
  unsigned uiResult = cQuery->NextPlan(uiPlan);
  cBlob->Set(&uiResult, sizeof(unsigned));
}

static void FnListCity(CQuery *cQuery, CBlob *cBlob)
{
  std::string strCity;
  unsigned uiResult = cQuery->ListCity((const char *)cBlob->DataPointer() + sizeof(unsigned), &strCity) ? 1 : 0;
  // dlka = vsledek + jmno
  unsigned uiLength = sizeof(unsigned) + (unsigned)strCity.length() + 1;
  char *pcData = new char [uiLength];
  // nvratov hodnota
  memcpy(pcData, &uiResult, sizeof(unsigned));
  // vstupn parametry
  strcpy(pcData + sizeof(unsigned), strCity.c_str());
  cBlob->Set(pcData, uiLength);
  delete [] pcData;
}

static void FnListAlias(CQuery *cQuery, CBlob *cBlob)
{
  std::string strAlias;
  std::string strCity;
  unsigned uiResult = cQuery->ListAlias((const char *)cBlob->DataPointer() + sizeof(unsigned), &strAlias, &strCity) ? 1 : 0;
  // dlka = vsledek + alias + jmno
  unsigned uiLength = sizeof(unsigned) + (unsigned)strAlias.length() + 1 + (unsigned)strCity.length() + 1;
  char *pcData = new char [uiLength];
  // nvratov hodnota
  memcpy(pcData, &uiResult, sizeof(unsigned));
  unsigned uiOffset = sizeof(unsigned);
  // vstupn parametry
  strcpy(pcData + uiOffset, strAlias.c_str());
  uiOffset += (unsigned)strAlias.length() + 1;
  strcpy(pcData + uiOffset, strCity.c_str());
  cBlob->Set(pcData, uiLength);
  delete [] pcData;
}

typedef void (*FnQuery)(CQuery *cQuery, CBlob *cBlob);

// poad ukazatel na funkce v tabulce a jejich poet mus souhlasit s vtovm typem CCommand::EQueryCmd
static const FnQuery afnTable[] = {
  FnAddCity,
  FnAddCityAlias,
  FnDeleteCity,
  FnFindCity,
  FnCityInfo,
  FnAddBusPlan,
  FnDeleteBusPlan,
  FnFindBusPlan,
  FnBusPlanInfo,
  FnAddBus,
  FnDeleteBus,
  FnFindBus,
  FnBusInfo,
  FnEnableBus,
  FnDisableBus,
  FnNewCustomer,
  FnAskSeats,
  FnLockSeats,
  FnUnlockSeats,
  FnNextBus,
  FnNextCity,
  FnNextPlan,
  FnListCity,
  FnListAlias,
};

unsigned  CServerQuery::ServerCall(void *pParam, CBlob *cBlob)
{
  CQuery *cQuery = reinterpret_cast<CServerQuery *>(pParam)->m_cQuery;
  if(cQuery == NULL)
    return 1; // nen pipojena databze
  unsigned uiCommand;
  cBlob->CopyData(&uiCommand, sizeof(unsigned), 0);
  if(uiCommand >= sizeof(afnTable) / sizeof(FnQuery))
    return 2; // nesprvn pkaz
  else { // zpracovn pkazu pomoc pslun funkce
    afnTable[uiCommand](cQuery, cBlob);
    return 0;
  }
}
