#include <winsock2.h> // mus bt ped <windows.h>
#include <windows.h>
#include <process.h>

#pragma hdrstop

#include "ServerTcp.h"
#include "../Common/SocketEndpoint.h"
#include "../Common/Command.h"
#include "../Common/UtilClass.h"

const unsigned MIN_THREADS = 1;
const unsigned MAX_THREADS = 50;

CServerTcp::CServerTcp(unsigned uiThreads, const char *pszHost, FnProcess fnProcess, void *pParam) :
  CServerConnection(fnProcess, pParam),
  CSocketEndpoint(pszHost)
{
  m_hThread = NULL;
  m_hTerminateEvent = CreateEvent(NULL, TRUE, FALSE, NULL); // nesignalizovan manual-reset udlost
  m_hRunEvent = CreateEvent(NULL, FALSE, FALSE, NULL); // nesignalizovan auto-reset udlost
  if(uiThreads < MIN_THREADS)
    uiThreads = MIN_THREADS;
  else if(uiThreads > MAX_THREADS)
    uiThreads = MAX_THREADS;
  m_uiThreads = uiThreads;
  m_hThreadCounter = CreateSemaphore(NULL, uiThreads, uiThreads, NULL); // potn zpracovatelskch vlken
}

CServerTcp::~CServerTcp()
{
  if(m_hThread != NULL)
    Stop(); // zastaven bcho serveru
  CloseHandle(m_hTerminateEvent);
  CloseHandle(m_hThreadCounter);
  CloseHandle(m_hRunEvent);
}

bool CServerTcp::Start(void)
{
  sockaddr_in rMyAddress;

  if(m_hThread == NULL) {
    OpenSocket();
    // naslouchej na zvolen IP adrese a portu
    Address(&rMyAddress);
    bind(Socket(), (const sockaddr *)&rMyAddress, sizeof(sockaddr_in));
    if(listen(Socket(), m_uiThreads) != SOCKET_ERROR) {
      ResetEvent(m_hTerminateEvent); // umonn innosti pracovnho vlkna
      m_hThread = (HANDLE)_beginthreadex(NULL, 0, WorkThread, this, 0, NULL); // odstartuj pracovn vlkno serveru
      return true;
    }
    // nastala njak chyba (oteven soketu, bind nebo listen)
    CloseSocket();
  }
  return false;
}

void CServerTcp::Stop(void)
{
  if(m_hThread != NULL) {
    SetEvent(m_hTerminateEvent);
    WaitForSingleObject(m_hThread, INFINITE); // ekn na ukonen pracovnho vlkna serveru
    CloseHandle(m_hThread);
    m_hThread = NULL;
    WaitForThreads(); // ekn na dokonen vech zpracovatelskch vlken
    CloseSocket();
  }
}

// pracovn vlkno serveru
unsigned int WINAPI CServerTcp::WorkThread(LPVOID pThreadParams)
{
  CServerTcp *cThis;
  timeval rTimeout = {0, 100000}; // 100 ms
  fd_set rReadSet;
  HANDLE hThread;

  cThis = (CServerTcp *)pThreadParams;
  FD_ZERO(&rReadSet);
  while(true) {
    if(WaitForSingleObject(cThis->m_hTerminateEvent, 0) == WAIT_OBJECT_0) // poadavek na ukonen?
      return 0;
    // test, zda na portu je poadavek na pipojen
    FD_SET(cThis->Socket(), &rReadSet);
    if(select(0, &rReadSet, NULL, NULL, &rTimeout) > 0) {
      if(WaitForSingleObject(cThis->m_hThreadCounter, 10) == WAIT_OBJECT_0) { // vlkna nejsou vyerpna?
        hThread = (HANDLE)_beginthreadex(NULL, 0, CmdThread, cThis, 0, NULL); // zpracovn ve zvltnm vlkn
        CloseHandle(hThread);
        WaitForSingleObject(cThis->m_hRunEvent, INFINITE); // ekn na pjem spojen zpracovatelskm vlknem 
      }
    }
    Sleep(0);
  }
}

// vlkno pro zpracovn pkazu
unsigned int WINAPI CServerTcp::CmdThread(LPVOID pThreadParams)
{
  CServerTcp *cThis;
  sockaddr_in rClientAddr;
  int iLength;
  SOCKET hClient;

  cThis = (CServerTcp *)pThreadParams;
  iLength = sizeof(sockaddr_in);
  // zskn naslouchajcho soketu pro tohoto klienta
  hClient = accept(cThis->Socket(), (sockaddr *)&rClientAddr, &iLength);
  SetEvent(cThis->m_hRunEvent); // oznmen pijet spojen
  if(hClient != INVALID_SOCKET) {
    cThis->CmdProcessing(hClient); // zpracovn pkazu
    closesocket(hClient); // uzaven soketu
  }
  ReleaseSemaphore(cThis->m_hThreadCounter, 1, NULL); // vlkno kon, inkrementace tae
  return 0;
}

void CServerTcp::CmdProcessing(SOCKET hClient)
{
  BYTE *pbInData;
  unsigned uiStatus;
  unsigned uiDataLen;
  bool fResult;
  unsigned uiResult;
  CBlob *cBlob;

  uiStatus = CMD_COMMFAIL_FLAG;
  uiDataLen = 0;
  cBlob = new CBlob;
  // pijat data maj strukturu: id (unsigned), dlka dat (unsigned), data
  if(::Receive(hClient, (char *)&uiStatus, sizeof(unsigned)) == sizeof(unsigned) &&
      ::Receive(hClient, (char *)&uiDataLen, sizeof(unsigned)) == sizeof(unsigned)) {
    if(uiDataLen != 0) {
      pbInData = new BYTE [uiDataLen];
      if(::Receive(hClient, (char *)pbInData, uiDataLen) == uiDataLen)
        cBlob->Set(pbInData, uiDataLen); // uloen sprvn pijatch dat
      else {
        uiStatus = CMD_COMMFAIL_FLAG; // pznak chyby komunikace
        uiDataLen = 0;
      }
      delete [] pbInData;
    }
    // zavolej zpracovn pkazu
    if(uiStatus != CMD_COMMFAIL_FLAG) {
      fResult = CallProcessing(&uiResult, cBlob);
      uiStatus = CMD_SETFUNCRESULT(uiResult) + (uiStatus & CMD_COUNTER_MASK);
      if(fResult)
        uiDataLen = cBlob->DataLen();
      else
        uiStatus |= CMD_FUNCFAIL_FLAG;
    }
  }
  // odesln odpovdi
  ::Send(hClient, (const char *)&uiStatus, sizeof(unsigned)); // id
  ::Send(hClient, (const char *)&uiDataLen, sizeof(unsigned)); // dlka dat
  if(uiDataLen != 0)
    ::Send(hClient, (const char *)cBlob->DataPointer(), uiDataLen); // data
  delete cBlob;
}

void CServerTcp::WaitForThreads(void)
{
  unsigned uiLastCount;
  unsigned uiFinish;

  uiFinish = m_uiThreads - 1;
  // ekn na dokonen vech vlken zpracovn pkaz
  while(true) {
    WaitForSingleObject(m_hThreadCounter, INFINITE); // -1
    ReleaseSemaphore(m_hThreadCounter, 1, (LPLONG)&uiLastCount); // +1
    if(uiLastCount == uiFinish)
      return;
    Sleep(100);
  }
}
