#ifndef _SERVERTCP_H
#define _SERVERTCP_H

#include "ServerConn.h"
#include "../Common/SocketEndpoint.h"

/*!
@file ServerTcp.h
Deklarace tdy @ref CServerTcp.
*/

//! serverov st TCP komunikace
/*!
Tda zajiuje serverovou st komunikace pomoc TCP protokolu. Implementuje rozhran 
@ref CServerConnection a pro prci se sokety vyuv funcionalitu @ref CSocketEndpoint.

Pi aktivovn serveru (vyvolnm @c Start) se vytvo pracovn vlkno, kter nastav
vnitn soket do naslouchacho stavu a ek na pchoz spojen. Jakmile se tak stane,
vytvo zpracovatelsk vlkno, kter si pro toto spojen vytvo klientsk soket, na kterm
dle komunikuje, zajiuje sprvn voln uivatelsk funkce a nakonec spojen uzave.
Pracovn vlkno serveru mezitm me ekat na dal pchoz spojen, pro nj vytvoit dal
zpracovatelsk vlkno, atd. Zpracovatelskch vlken me pracovat ada paraleln, take 
ppadn del zpracovvn pkaz neblokuje innost dalch klient.
@image html ServerTcp.png "Vlkna pi innosti TCP serveru"
Na diagramu je volajc vlkno zobrazeno lutmi bloky, pracovn vlkno serveru pak modrmi
bloky a jeden reprezentant zpracovatelskho vlkna barvou oranovou.

Aplikace me spoutt vce instanc TCP server, kter pracuj na rznch IP adresch i
portech. Jednotliv instance se navzjem neovlivuj.
*/
class CServerTcp : public CServerConnection, private CSocketEndpoint {
public:
  //! konstruktor
  /*!
  Vytvoen instance vicevlknovho TCP serveru.
  @param[in] uiThreads maximln poet paraleln bcch zpracovatelskch vlken
  @param[in] pszHost uren IP adresy a portu pro naslouchac soket
  @n U serveru je mon naslouchat na vech IP adresch, kter m pota pidleny a pijmat
  tak zprvy od vech vzdlench i loklnch klient. Podporovan formty jsou popsny u
  @ref CSocketEndpoint::CSocketEndpoint.
  @param[in] fnProcess adresa funkce typu @ref FnProcess pro zpracovn pkaz na serveru
  @param[in] pParam libovoln parametr, kter bude pedn uivatelsk funkci
  */
  CServerTcp(unsigned uiThreads, const char *pszHost, FnProcess fnProcess, void *pParam);
  //! destruktor
  /*! ek na dokonen vech vlken, pak uvoln veker alokovan zdroje. */
  virtual ~CServerTcp();
  virtual bool Start(void);
  virtual void Stop(void);
protected:
  //! komunikace a zpracovn pkazu
  /*!
  Pi pjmu spojen je vytvoen klientsk soket. Server nsledn vol tuto metodu, kter mus
  zajistit pjem zprvy, jej zpracovn pomoc uivatelsk funkce a nakonec odesln vsledk
  zpracovn dat zpt klentovi.
  @param[in] hClient klientsk soket zskan pi pjmu spojen (@c accept)
  */
  virtual void CmdProcessing(SOCKET hClient);
private:
  HANDLE m_hThread;
  unsigned m_uiThreads;
  HANDLE m_hTerminateEvent;
  HANDLE m_hThreadCounter;
  HANDLE m_hRunEvent;
  static unsigned int WINAPI WorkThread(LPVOID pThreadParams);
  static unsigned int WINAPI CmdThread(LPVOID pThreadParams);
  void WaitForThreads(void);
};

#endif
